<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="2.0"><!--
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"     xsi:schemaLocation="
        urn:us:mil:ces:metadata:ddms:5 ../DDMS-v5.xsd
        urn:us:gov:ic:tdf ../../IC-TDF/Schemas/IC-TDF.xsd"

        DDMS 5 Alpha release
        
        This XSL Transform is provided as part of the DoD Metadata Specification (DDMS) XML version 5 alpha.  The intent of
        this release is to provide the community with a preview of the proposed changes for DDMS 5 prior to voting.  No changes
        in this alpha release have been officially adopted or voted on as normative.
        
        Feedback is vital to this process; please make use of the DDMS mailing list < ddms@fgm.com > to discuss the contents
        of this alpha release and any suggestions or problems you may have, or errors you may find. 
    --><!--
        Notice:
           This transform is intended to be useful and informative, but is not guaranteed to generate semantically-valid
        TDF XML for all purposes.  In particular, at several points the XSLT attempts to make a 'best guess' for data
        that is required but may be ambiguously specified in or absent from the original DDMS.
           At a minimum, the 'User-Editable Variables' section below should be edited by the user(s) of this XSLT in order
        to generate TDF XML that is line with the expected norm for their organization, COI, or other community using DDMS 5.
        Additionally, further processing in the form of XSLTs or programs may be needed to ensure that the output is both
        syntactically and semantically valid, that is, that it 'makes sense' in both what it is saying and how it is
        saying it.
           All TDF XML should be validated against all appropriate Schemas and Schematron to assure a baseline of conformity
        to the standards.  Transformed XML may be valid without making sense; human review is best to assure appropriateness
        of transformations.
     --><!-- ==================================== ->
            User-Editable Variables
    <-  ==================================== --><!-- This assumes the DDMS is stand-alone and not packaged with the described data object.
         Set this to be false if the TDO is to contain the data object. -->
   <xsl:variable name="external" select="true()"/>
   <xsl:variable name="guideForTDO">GUIDE://0000/0003</xsl:variable>
   <xsl:variable name="guideForResource">GUIDE://0000/0001</xsl:variable>
   <xsl:variable name="guideForDiscovery">GUIDE://0000/0002</xsl:variable>
   <xsl:variable name="responsibleOrganization">NSA</xsl:variable>
   <!-- This should be the organization running this XSLT to generate the TDO -->
   <xsl:variable name="organization">
      <edh:ResponsibleEntity xmlns:edh="urn:us:gov:ic:edh">
         <edh:Country>USA</edh:Country>
         <edh:Organization>
            <xsl:value-of select="$responsibleOrganization"/>
         </edh:Organization>
      </edh:ResponsibleEntity>
   </xsl:variable>
   <!-- This uses an XSLT implementation of a UUID generator to create IDs for resultant TDOs.
        Replace with something else if appropriate. -->
   <xsl:function name="ddms:tdo-Id" as="xs:string">
      <xsl:value-of select="concat($tdo-id-prefix, uuid:get-uuid())"/>
   </xsl:function>
   <!-- Prefix for generated UUIDs -->
   <xsl:variable name="tdo-id-prefix" select="'uuid:test:'"/>
   <!-- ==================================== ->
        End of User-Editable Variables
    <-  ==================================== -->
   <xsl:variable name="externalPrefix"
                 select="substring('External',0,number($external) * 9)"/>
   <xsl:template match="/"><!-- Preamble; create a TDO. --><?oxygen SCHSchema="../../Schematron/IC-TDF/IC-TDF_XML.sch"?><?oxygen SCHSchema="../../Schematron/IC-EDH/IC-EDH_XML.sch"?><?oxygen SCHSchema="../../Schematron/ISM/ISM_XML.sch"?><?oxygen SCHSchema="../../Schematron/ARH/ARH_XML.sch"?><?oxygen SCHSchema="../../Schematron/NTK/NTK_XML.sch"?>
      <tdf:TrustedDataObject xmlns:tdf="urn:us:gov:ic:tdf" tdf:version="testversion">
         <tdf:HandlingAssertion tdf:scope="TDO">
            <tdf:HandlingStatement>
               <edh:Edh xmlns:edh="urn:us:gov:ic:edh"
                        xmlns:arh="urn:us:gov:ic:arh"
                        edh:DESVersion="1"
                        arh:DESVersion="1"><!-- apply ism and ntk DESVersions from ddms4:resource -->
                  <xsl:apply-templates select="/ddms4:resource/@*" mode="EDH"/>
                  <xsl:choose>
                     <xsl:when test="starts-with(ddms4:resource/ddms4:identifier/@ddms:value,'GUIDE://')">
                        <edh:Identifier>
                           <xsl:value-of select="ddms4:resource/ddms4:identifier/@ddms:value"/>
                        </edh:Identifier>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:comment>All identifiers must be GUIDE identifiers.  Replace this stub with a valid GUIDE</xsl:comment>
                        <edh:Identifier>
                           <xsl:value-of select="$guideForTDO"/>
                        </edh:Identifier>
                     </xsl:otherwise>
                     <!--xsl:value-of select="ddms:tdo-Id()"/-->
                  </xsl:choose>
                  <edh:DataItemCreateDateTime><!--xsl:value-of select="substring-before(string(adjust-dateTime-to-timezone(current-dateTime(),xs:dayTimeDuration('PT0H'))),'.')" />Z -->
                     <xsl:choose>
                        <xsl:when test="/ddms4:resource/@ism:createDate">
                           <xsl:variable name="dt">
                              <xsl:value-of select="/ddms4:resource/@ism:createDate"/>
                           </xsl:variable>
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat($dt,'T23:59:59')) - xs:dayTimeDuration('P1D'),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                        <xsl:otherwise>
                           <xsl:variable name="dt">
                              <xsl:value-of select="substring-before(string(current-dateTime()),'T')"/>
                           </xsl:variable>
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat($dt,'T23:59:59')) - xs:dayTimeDuration('P1D'),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:otherwise>
                     </xsl:choose>
                  </edh:DataItemCreateDateTime>
                  <xsl:choose>
                     <xsl:when test="$organization">
                        <xsl:copy-of select="$organization"/>
                     </xsl:when>
                     <xsl:otherwise><!-- Best Guess -->
                        <xsl:choose>
                           <xsl:when test="/ddms4:resource/ddms4:metacardInfo/ddms4:creator/ddms4:organization">
                              <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:creator/ddms4:organization[1]"
                                                   mode="EDH"/>
                           </xsl:when>
                           <xsl:when test="/ddms4:resource/ddms4:metacardInfo/ddms4:publisher/ddms4:organization">
                              <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:publisher/ddms4:organization[1]"
                                                   mode="EDH"/>
                           </xsl:when>
                        </xsl:choose>
                     </xsl:otherwise>
                  </xsl:choose>
                  <arh:Security>
                     <xsl:apply-templates select="//ddms4:resource/@ism:*" mode="ARH"/>
                     <xsl:if test="//ddms4:resource/@ism:classification!='U' and not(//ddms4:resource/@ism:derivativelyClassifiedBy)">
                        <xsl:if test="not(//ddms4:resource/@ism:derivedFrom)">
                           <xsl:attribute name="ism:derivedFrom">XXX</xsl:attribute>
                        </xsl:if>
                        <xsl:attribute name="ism:derivativelyClassifiedBy">GSH</xsl:attribute>
                     </xsl:if>
                     <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ntk:Access" mode="ARH"/>
                     <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:noticeList" mode="ARH"/>
                  </arh:Security>
               </edh:Edh>
            </tdf:HandlingStatement>
         </tdf:HandlingAssertion>
         <tdf:HandlingAssertion tdf:scope="PAYL">
            <tdf:HandlingStatement><!--xsl:element namespace="urn:us:gov:ic:edh"
                        name="{concat($externalPrefix, 'Edh')}"-->
               <xsl:element name="edh:ExternalEdh"><!--xsl:apply-templates select="/ddms4:resource/@ism:*[local-name()='DESVersion']"/>
                            <xsl:apply-templates select="/ddms4:resource/@ntk:*[local-name()='DESVersion']"/-->
                  <xsl:attribute name="arh:DESVersion">1</xsl:attribute>
                  <xsl:attribute name="edh:DESVersion">1</xsl:attribute>
                  <xsl:attribute name="ism:DESVersion">9</xsl:attribute>
                  <xsl:attribute name="ntk:DESVersion">7</xsl:attribute>
                  <xsl:apply-templates select="/ddms4:resource/ddms4:security" mode="EDH"/>
                  <xsl:choose>
                     <xsl:when test="starts-with(.,'GUIDE://')">
                        <edh:Identifier xmlns:edh="urn:us:gov:ic:edh">
                           <xsl:value-of select="."/>
                        </edh:Identifier>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:comment>All identifiers must be GUIDE identifiers.  Replace this stub with a valid GUIDE</xsl:comment>
                        <edh:Identifier xmlns:edh="urn:us:gov:ic:edh">
                           <xsl:value-of select="$guideForResource"/>
                        </edh:Identifier>
                     </xsl:otherwise>
                     <!--xsl:value-of select="ddms:tdo-Id()"/-->
                  </xsl:choose>
                  <!--edh:Identifier>
              <xsl:choose>
                <xsl:when test="starts-with(.,'GUIDE://')">
                  <xsl:value-of select="." />
                </xsl:when>
                <xsl:otherwise>
                  <xsl:value-of select="$guideForResource" />
                </xsl:otherwise>
              </xsl:choose>
            </edh:Identifier-->
                  <edh:DataItemCreateDateTime xmlns:edh="urn:us:gov:ic:edh">
                     <xsl:choose><!-- Glenda -->
                        <xsl:when test="/ddms4:resource/ddms4:dates/@ddms4:created[string-length(.) = 10]">
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat(/ddms4:resource/ddms4:dates/@ddms4:created,'T00:00:00Z')),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                        <xsl:when test="/ddms4:resource/ddms4:dates/@ddms4:created[string-length(.) gt 15]">
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(/ddms4:resource/ddms4:dates/@ddms4:created),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                        <xsl:when test="/ddms4:resource/ddms4:dates/@ddms4:created[string-length(.) lt 15]">
                           <xsl:variable name="dt">
                              <xsl:value-of select="substring-before(/ddms4:resource/ddms4:dates/@ddms4:created,'T')"/>
                           </xsl:variable>
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat($dt,'T00:00:00Z')),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                        <xsl:otherwise>1901-01-01T00:00:00Z</xsl:otherwise>
                     </xsl:choose>
                  </edh:DataItemCreateDateTime>
                  <!--xsl:value-of select="."/-->
                  <xsl:choose>
                     <xsl:when test="/ddms4:resource/ddms4:creator[ddms4:organization][1]">
                        <xsl:apply-templates select="/ddms4:resource/ddms4:creator[ddms4:organization][1]"
                                             mode="EDH"/>
                     </xsl:when>
                     <xsl:when test="/ddms4:resource/ddms4:publisher[ddms4:organization][1]">
                        <xsl:apply-templates select="/ddms4:resource/ddms4:publisher[ddms4:organization][1]"
                                             mode="EDH"/>
                     </xsl:when>
                  </xsl:choose>
                  <xsl:apply-templates select="/ddms4:resource/ddms4:security" mode="ARH"/>
               </xsl:element>
               <xsl:text/>
            </tdf:HandlingStatement>
         </tdf:HandlingAssertion>
         <tdf:Assertion tdf:scope="PAYL" tdf:type="DISCOVERY">
            <tdf:StatementMetadata>
               <edh:Edh xmlns:edh="urn:us:gov:ic:edh"
                        xmlns:arh="urn:us:gov:ic:arh"
                        edh:DESVersion="1"
                        arh:DESVersion="1">
                  <xsl:apply-templates select="/ddms4:resource/@*" mode="EDH"/>
                  <xsl:choose>
                     <xsl:when test="starts-with(.,'GUIDE://')">
                        <edh:Identifier>
                           <xsl:value-of select="."/>
                        </edh:Identifier>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:comment>All identifiers must be GUIDE identifiers.  Replace this stub with a valid GUIDE</xsl:comment>
                        <edh:Identifier>
                           <xsl:value-of select="$guideForDiscovery"/>
                        </edh:Identifier>
                     </xsl:otherwise>
                  </xsl:choose>
                  <xsl:variable name="dt1">
                     <xsl:value-of select="/ddms4:resource/@ism:createDate"/>
                  </xsl:variable>
                  <xsl:variable name="dt2">
                     <xsl:value-of select="substring(/ddms4:resource/ddms4:metacardInfo/ddms4:dates/@ddms4:created,1,10)"/>
                  </xsl:variable>
                  <!--xsl:if test="/ddms4:resource/ddms4:metacardInfo/ddms4:dates/@ddms4:created and /ddms4:resource/@ism:createDate and xs:date(/ddms4:resource/ddms4:metacardInfo/ddms4:dates/@ddms4:created) gt xs:date(/ddms4:resource/@ism:createDate)"-->
                  <xsl:if test="$dt2 and $dt1 and xs:date($dt2) gt xs:date($dt1)">
                     <xsl:comment>Error in original file regarding the dates.  Resource metacard date must not be later than Metacard security date, if present.  Resource security date has been substituted here.</xsl:comment>
                  </xsl:if>
                  <edh:DataItemCreateDateTime>
                     <xsl:choose>
                        <xsl:when test="$dt2 and $dt1 and xs:date($dt2) gt xs:date($dt1)">
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat($dt1,'T23:59:59')) - xs:dayTimeDuration('P1D'),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                        <xsl:when test="$dt2 and $dt1 and xs:date($dt2) &lt;= xs:date($dt1)">
                           <xsl:value-of select="adjust-dateTime-to-timezone(xs:dateTime(concat($dt2,'T23:59:59')) - xs:dayTimeDuration('P1D'),xs:dayTimeDuration('PT0H'))"/>
                        </xsl:when>
                     </xsl:choose>
                  </edh:DataItemCreateDateTime>
                  <xsl:choose>
                     <xsl:when test="/ddms4:resource/ddms4:metacardInfo/ddms4:creator/ddms4:organization">
                        <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:creator[ddms4:organization][1]"
                                             mode="EDH"/>
                     </xsl:when>
                     <xsl:when test="/ddms4:resource/ddms4:metacardInfo/ddms4:publisher/ddms4:organization">
                        <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:publisher[ddms4:organization][1]"
                                             mode="EDH"/>
                     </xsl:when>
                  </xsl:choose>
                  <arh:Security>
                     <xsl:apply-templates select="/ddms4:resource/@ism:*"/>
                     <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ntk:Access" mode="ARH"/>
                     <xsl:apply-templates select="/ddms4:resource/ddms4:metacardInfo/ddms4:noticeList" mode="ARH"/>
                  </arh:Security>
               </edh:Edh>
            </tdf:StatementMetadata>
            <tdf:StructuredStatement>
               <xsl:apply-templates select="/ddms4:resource" mode="DDMS"/>
            </tdf:StructuredStatement>
         </tdf:Assertion>
         <xsl:choose>
            <xsl:when test="$external">
               <xsl:element name="tdf:ReferenceValuePayload">
                  <xsl:attribute name="tdf:uri">
                     <xsl:value-of select="/ddms4:resource/ddms4:identifier[@ddms4:qualifer='URL']"/>
                  </xsl:attribute>
               </xsl:element>
               <xsl:text/>
            </xsl:when>
            <xsl:otherwise>
               <tdf:StructuredPayload>
                  <xsl:comment>Payload contents would be added in here, possibly by another operation.</xsl:comment>
               </tdf:StructuredPayload>
            </xsl:otherwise>
         </xsl:choose>
      </tdf:TrustedDataObject>
   </xsl:template>
   <!-- EDH conversions -->
   <xsl:template match="ddms4:publisher" mode="EDH">
      <edh:ResponsibleEntity xmlns:edh="urn:us:gov:ic:edh"><!-- best guess -->
         <edh:Country>USA</edh:Country>
         <edh:Organization>
            <xsl:value-of select="ddms4:organization/ddms4:name"/>
         </edh:Organization>
         <xsl:choose>
            <xsl:when test="child::ddms4:subOrganization">
               <edh:SubOrganization>
                  <xsl:value-of select="translate(child::ddms4:subOrganization,' ','')"/>
               </edh:SubOrganization>
            </xsl:when>
         </xsl:choose>
      </edh:ResponsibleEntity>
   </xsl:template>
   <xsl:template match="ddms4:creator" mode="EDH">
      <edh:ResponsibleEntity xmlns:edh="urn:us:gov:ic:edh"><!-- best guess -->
         <edh:Country>USA</edh:Country>
         <edh:Organization>
            <xsl:value-of select="ddms4:organization/ddms4:name"/>
         </edh:Organization>
         <xsl:choose>
            <xsl:when test="child::ddms4:subOrganization">
               <edh:SubOrganization>
                  <xsl:value-of select="translate(child::ddms4:subOrganization,' ','')"/>
               </edh:SubOrganization>
            </xsl:when>
         </xsl:choose>
      </edh:ResponsibleEntity>
   </xsl:template>
   <xsl:template match="ddms4:resource/@*[contains(local-name(), 'DES')]" mode="EDH">
      <xsl:attribute name="ism:DESVersion">9</xsl:attribute>
      <xsl:attribute name="ntk:DESVersion">7</xsl:attribute>
      <!--xsl:copy-of select="."/-->
   </xsl:template>
   <!-- DEBUG TRACE 
    <xsl:template match="*|@*" mode="EDH" priority="-1">
        <xsl:comment>EDH ignoring <xsl:value-of select="concat(namespace-uri(), ':', name())"/></xsl:comment>
    </xsl:template>
-->
   <xsl:template match="*|@*|node()" mode="EDH" priority="-2">
      <xsl:apply-templates select="@ism:DESVersion"/>
   </xsl:template>
   <!-- snarf -->
   <!-- ARH conversions -->
   <xsl:template match="ddms4:security" mode="ARH">
      <xsl:element name="arh:ExternalSecurity">
         <xsl:attribute name="arh:DESVersion">1</xsl:attribute>
         <xsl:apply-templates select="@ism:*"/>
         <xsl:apply-templates select="ntk:Access" mode="ARH"/>
         <xsl:apply-templates select="ddms4:noticeList" mode="ARH"/>
      </xsl:element>
   </xsl:template>
   <xsl:template match="ddms4:security/ddms4:noticeList" mode="ARH">
      <xsl:element name="ism:NoticeExternalList">
         <xsl:apply-templates select="@ism:*"/>
         <xsl:for-each select="ism:Notice">
            <xsl:element name="ism:NoticeExternal">
               <xsl:choose>
                  <xsl:when test="$external">
                     <xsl:attribute name="ism:externalNotice">true</xsl:attribute>
                  </xsl:when>
               </xsl:choose>
               <xsl:apply-templates select="@ism:*"/>
               <xsl:apply-templates select="*"/>
            </xsl:element>
            <xsl:text/>
         </xsl:for-each>
      </xsl:element>
   </xsl:template>
   <xsl:template match="ddms4:noticeList" mode="ARH">
      <ism:NoticeList xmlns:ism="urn:us:gov:ic:ism">
         <xsl:apply-templates select="@ism:*"/>
         <xsl:apply-templates select="//ism:Notice"/>
      </ism:NoticeList>
   </xsl:template>
   <xsl:template match="ism:Notice">
      <ism:Notice xmlns:ism="urn:us:gov:ic:ism">
         <xsl:apply-templates select="@ism:*"/>
         <xsl:if test="../../../@ism:atomicEnergyMarkings[.='FRD' or .='RD']">
            <xsl:if test=".//ddms4:noticeList/ism:Notice">
               <xsl:attribute name="ism:noticeType1">FRD</xsl:attribute>
            </xsl:if>
         </xsl:if>
         <!--xsl:copy-of select="ism:Notice"/-->
         <xsl:apply-templates select="*"/>
      </ism:Notice>
   </xsl:template>
   <xsl:template match="ism:NoticeText">
      <ism:NoticeText xmlns:ism="urn:us:gov:ic:ism">
         <xsl:apply-templates select="@ism:*"/>
         <xsl:apply-templates select="@ntk:*"/>
         <xsl:value-of select="."/>
      </ism:NoticeText>
   </xsl:template>
   <xsl:template match="ddms4:security/ntk:Access" mode="ARH">
      <xsl:element name="ntk:ExternalAccess" ism:classification="U" ism:ownerProducer="USA">
         <xsl:choose>
            <xsl:when test="$external">
               <xsl:attribute name="ntk:externalReference">true</xsl:attribute>
            </xsl:when>
         </xsl:choose>
         <xsl:apply-templates select="@ism:*"/>
         <xsl:copy-of select="*"/>
      </xsl:element>
      <xsl:text/>
   </xsl:template>
   <xsl:template match="ntk:Access" mode="ARH">
      <xsl:copy/>
   </xsl:template>
   <!-- DEBUG TRACE
    <xsl:template match="*|@*" mode="ARH" priority="-1">
        <xsl:comment>ARH ignoring <xsl:value-of select="concat(namespace-uri(), ':', name())"/></xsl:comment>
    </xsl:template>
 -->
   <xsl:template match="node()" mode="ARH" priority="-2"/>
   <!-- snarf -->
   <!-- DDMS trimming -->
   <xsl:template match="ddms4:resource" mode="DDMS">
      <ddms:resource xmlns:ddms="urn:us:mil:ces:metadata:ddms:5">
         <xsl:attribute name="ism:DESVersion">9</xsl:attribute>
         <xsl:attribute name="ntk:DESVersion">7</xsl:attribute>
         <xsl:apply-templates select="*" mode="DDMS"/>
      </ddms:resource>
   </xsl:template>
   <xsl:template match="ddms4:security|ddms4:noticeList" mode="DDMS" priority="2"/>
   <!-- trimming out -->
   <!-- Upgrade DDMS 4 elements to DDMS 5 elements -->
   <xsl:template match="ddms4:*" mode="DDMS">
      <xsl:element name="ddms:{local-name()}">
         <xsl:apply-templates select="@*"/>
         <xsl:apply-templates select="*|text()" mode="DDMS"/>
      </xsl:element>
   </xsl:template>
   <!-- Upgrade ddms attributes to ddms4 attributes -->
   <xsl:template match="@ddms4:*">
      <xsl:attribute name="ddms:{local-name()}">
         <xsl:value-of select="."/>
      </xsl:attribute>
   </xsl:template>
   <xsl:template match="@ism:DESVersion">
      <xsl:attribute name="ism:DESVersion">9</xsl:attribute>
   </xsl:template>
   <xsl:template match="@ntk:DESVersion">
      <xsl:attribute name="ntk:DESVersion">7</xsl:attribute>
   </xsl:template>
   <xsl:template match="@ddms4:POCType">
      <xsl:attribute name="ism:pocType">
         <xsl:value-of select="."/>
      </xsl:attribute>
   </xsl:template>
   <xsl:template match="@*" mode="ARH">
      <xsl:if test="../@ism:resourceElement and (../@ism:atomicEnergyMarkings='FRD' or ../@ism:atomicEnergyMarkings='RD')">
         <xsl:attribute name="ism:declassException">AEA</xsl:attribute>
      </xsl:if>
      <xsl:attribute name="{local-name()}" namespace="{namespace-uri()}">
         <xsl:choose>
            <xsl:when test="local-name(.) = 'DESVersion' and namespace-uri(.)='urn:us:gov:ic:ism'">9</xsl:when>
            <xsl:when test="local-name(.) = 'DESVersion' and namespace-uri(.)='urn:us:gov:ic:ntk'">7</xsl:when>
            <xsl:when test="local-name()='createDate'">
               <xsl:attribute name="ism:createDate">
                  <xsl:choose>
                     <xsl:when test="/ddms4:resource/@ism:createDate">
                        <xsl:value-of select="/ddms4:resource/@ism:createDate"/>
                     </xsl:when>
                     <xsl:otherwise>
                        <xsl:value-of select="substring-before(string(current-dateTime()),'T')"/>
                     </xsl:otherwise>
                  </xsl:choose>
               </xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
               <xsl:value-of select="."/>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:attribute>
   </xsl:template>
   <xsl:template match="@*">
      <xsl:attribute name="{local-name()}" namespace="{namespace-uri()}">
         <xsl:value-of select="."/>
      </xsl:attribute>
   </xsl:template>
   <xsl:template match="ddms4:geospatialCoverage/ddms4:geographicIdentifier/ddms4:countryCode"
                 mode="DDMS">
      <xsl:element name="ddms:countryCode">
         <xsl:attribute name="ddms:qualifier">other</xsl:attribute>
         <xsl:attribute name="ddms:otherQualifier">
            <xsl:value-of select="@ddms4:qualifier"/>
         </xsl:attribute>
         <xsl:attribute name="ddms:value">
            <xsl:value-of select="@ddms4:value"/>
         </xsl:attribute>
      </xsl:element>
   </xsl:template>
   <xsl:template match="ddms4:geospatialCoverage/ddms4:geographicIdentifier/ddms4:subDivisionCode"
                 mode="DDMS">
      <xsl:choose>
         <xsl:when test="contains(.,'GENC')">
            <xsl:copy-of select="*"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:element name="ddms:countryCode">
               <xsl:attribute name="ddms:qualifier">other</xsl:attribute>
               <xsl:attribute name="ddms:otherQualifier">
                  <xsl:value-of select="@ddms4:qualifier"/>
               </xsl:attribute>
               <xsl:attribute name="ddms:value">
                  <xsl:value-of select="@ddms4:value"/>
               </xsl:attribute>
            </xsl:element>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>
   <!--
Functions in the uuid: namespace are used to calculate a UUID
The method used is a derived timestamp method, which is explained
here: http://www.famkruithof.net/guid-uuid-timebased.html
and here: http://www.ietf.org/rfc/rfc4122.txt
-->
   <!--
Returns the UUID
-->
   <xsl:function name="uuid:get-uuid" as="xs:string*">
      <xsl:variable name="ts" select="uuid:ts-to-hex(uuid:generate-timestamp())"/>
      <xsl:value-of separator="-"
                    select="             substring($ts, 8, 8),             substring($ts, 4, 4),             string-join((uuid:get-uuid-version(), substring($ts, 1, 3)), ''),             uuid:generate-clock-id(),             uuid:get-network-node()"/>
   </xsl:function>
   <!--
internal aux. fu
with saxon, this creates a more-unique result with
generate-id then when just using a variable containing a node
-->
   <xsl:function name="uuid:_get-node">
      <xsl:comment/>
   </xsl:function>
   <!-- generates some kind of unique id -->
   <xsl:function name="uuid:get-id" as="xs:string">
      <xsl:sequence select="generate-id(uuid:_get-node())"/>
   </xsl:function>
   <!--
should return the next nr in sequence, but this can't be done
in xslt. Instead, it returns a guaranteed unique number
-->
   <xsl:function name="uuid:next-nr" as="xs:integer">
      <xsl:variable name="node">
         <xsl:comment/>
      </xsl:variable>
      <xsl:sequence select="             xs:integer(replace(             generate-id($node), '\D', ''))"/>
   </xsl:function>
   <!-- internal fu for returning hex digits only -->
   <xsl:function name="uuid:_hex-only" as="xs:string">
      <xsl:param name="string"/>
      <xsl:param name="count"/>
      <xsl:sequence select="             substring(replace(             $string, '[^0-9a-fA-F]', '')             , 1, $count)"/>
   </xsl:function>
   <!-- may as well be defined as returning the same seq each time -->
   <xsl:variable name="_clock" select="uuid:get-id()"/>
   <xsl:function name="uuid:generate-clock-id" as="xs:string">
      <xsl:sequence select="uuid:_hex-only($_clock, 4)"/>
   </xsl:function>
   <!--
returns the network node, this one is 'random', but must
be the same within calls. The least-significant bit must be '1'
when it is not a real MAC address (in this case it is set to '1')
-->
   <xsl:function name="uuid:get-network-node" as="xs:string">
      <xsl:sequence select="uuid:_hex-only('09-17-3F-13-E4-C5', 12)"/>
   </xsl:function>
   <!-- returns version, for timestamp uuids, this is "1" -->
   <xsl:function name="uuid:get-uuid-version" as="xs:string">
      <xsl:sequence select="'1'"/>
   </xsl:function>
   <!--
Generates a timestamp of the amount of 100 nanosecond
intervals from 15 October 1582, in UTC time.
-->
   <xsl:function name="uuid:generate-timestamp"><!--
date calculation automatically goes
correct when you add the timezone information, in this
case that is UTC.
-->
      <xsl:variable name="duration-from-1582" as="xs:dayTimeDuration">
         <xsl:sequence select="                 current-dateTime() -                 xs:dateTime('1582-10-15T00:00:00.000Z')"/>
      </xsl:variable>
      <xsl:variable name="random-offset" as="xs:integer">
         <xsl:sequence select="uuid:next-nr() mod 10000"/>
      </xsl:variable>
      <!-- do the math to get the 100 nano second intervals -->
      <xsl:sequence select="             (days-from-duration($duration-from-1582) * 24 * 60 * 60 +             hours-from-duration($duration-from-1582) * 60 * 60 +             minutes-from-duration($duration-from-1582) * 60 +             seconds-from-duration($duration-from-1582)) * 1000             * 10000 + $random-offset"/>
   </xsl:function>
   <!-- simple non-generalized function to convert from timestamp to hex -->
   <xsl:function name="uuid:ts-to-hex">
      <xsl:param name="dec-val"/>
      <xsl:value-of separator=""
                    select="             for $i in 1 to 15             return (0 to 9, tokenize('A B C D E F', ' '))             [             $dec-val idiv             xs:integer(math:power(16, 15 - $i))             mod 16 + 1             ]"/>
   </xsl:function>
</xsl:stylesheet>
