<?xml version="1.0" encoding="UTF-8"?>
<!--UNCLASSIFIED--><xsl:stylesheet xmlns:xs="http://www.w3.org/2001/XMLSchema"
                xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                xmlns:saxon="http://saxon.sf.net/"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:schold="http://www.ascc.net/xml/schematron"
                xmlns:iso="http://purl.oclc.org/dsdl/schematron"
                xmlns:xhtml="http://www.w3.org/1999/xhtml"
                xmlns:ism="urn:us:gov:ic:ism"
                xmlns:irm="urn:us:gov:ic:irm"
                xmlns:ntk="urn:us:gov:ic:ntk"
                xmlns:tdf="urn:us:gov:ic:tdf"
                xmlns:icid="urn:us:gov:ic:id"
                xmlns:usagency="urn:us:gov:ic:usagency"
                xmlns:pm="urn:us:gov:ic:pm"
                xmlns:intdis="urn:us:gov:ic:intdis"
                xmlns:mime="urn:us:gov:ic:mime"
                xmlns:genc="urn:us:gov:ic:icgenc"
                xmlns:ddms="urn:us:mil:ces:metadata:ddms:5"
                xmlns:cve="urn:us:gov:ic:cve"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:dtf="date:time:function"
                xmlns:util="urn:us:gov:ic:irm:xsl:util"
                version="2.0"><!--Implementers: please note that overriding process-prolog or process-root is 
    the preferred method for meta-stylesheets to use where possible. -->
<xsl:param name="archiveDirParameter"/>
   <xsl:param name="archiveNameParameter"/>
   <xsl:param name="fileNameParameter"/>
   <xsl:param name="fileDirParameter"/>
   <xsl:variable name="document-uri">
      <xsl:value-of select="document-uri(/)"/>
   </xsl:variable>

   <!--PHASES-->


<!--PROLOG-->
<xsl:output xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
               method="xml"
               omit-xml-declaration="no"
               standalone="yes"
               indent="yes"/>

   <!--XSD TYPES FOR XSLT2-->


<!--KEYS AND FUNCTIONS-->
<xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="util:meetsType"
                 as="xs:boolean">
		    <xsl:param name="value"/>
		    <xsl:param name="typePattern" as="xs:string"/>
		    <xsl:value-of select="matches(string($value), concat('^(', $typePattern, ')$'))"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="util:containsAnyOfTheTokens"
                 as="xs:boolean">
		    <xsl:param name="attribute"/>
		    <xsl:param name="tokenList" as="xs:string+"/>
		    <xsl:value-of select="some $attrToken in tokenize(normalize-space(string($attribute)), ' ') satisfies $attrToken = $tokenList"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:getMaxDay"
                 as="xs:string">
		    <xsl:param name="date" as="xs:dateTime"/>
		    <xsl:variable name="month" select="number(dtf:getMonth(string($date)))"/>
		    <xsl:choose>
			      <xsl:when test="$month = (1,3,5,7,8,10,12)">
				        <xsl:value-of select="31"/>
			      </xsl:when>
			      <xsl:when test="$month = (4,6,9,11)">
				        <xsl:value-of select="30"/>
			      </xsl:when>
			      <xsl:otherwise>
				        <xsl:choose>
					          <xsl:when test="dtf:isLeapYear(string($date))">
						            <xsl:value-of select="29"/>
					          </xsl:when>
					          <xsl:otherwise>
						            <xsl:value-of select="28"/>
					          </xsl:otherwise>
				        </xsl:choose>
			      </xsl:otherwise>
		    </xsl:choose>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:isLeapYear"
                 as="xs:boolean">
		    <xsl:param name="date" as="xs:string"/>
		    <xsl:variable name="year" as="xs:integer" select="xs:integer(dtf:getYear($date))"/>
		    <xsl:choose>
			      <xsl:when test="$year mod 100 = 0">
				        <xsl:choose>
					          <xsl:when test="$year mod 400 = 0">
						            <xsl:value-of select="true()"/>
					          </xsl:when>
					          <xsl:otherwise>
						            <xsl:value-of select="false()"/>
					          </xsl:otherwise>
				        </xsl:choose>
			      </xsl:when>
			      <xsl:otherwise>
				        <xsl:choose>
					          <xsl:when test="$year mod 4 = 0">
						            <xsl:value-of select="true()"/>
					          </xsl:when>
					          <xsl:otherwise>
						            <xsl:value-of select="false()"/>
					          </xsl:otherwise>
				        </xsl:choose>
			      </xsl:otherwise>
		    </xsl:choose>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:replaceDateTimeDay"
                 as="xs:dateTime">
		    <xsl:param name="dateTime" as="xs:dateTime"/>
		    <xsl:param name="newDayString" as="xs:string"/>
		    <xsl:variable name="beforeDay" select="substring(string($dateTime), 1, 8)"/>
		    <xsl:variable name="afterDay" select="substring(string($dateTime), 11)"/>
		    <xsl:value-of select="concat($beforeDay, $newDayString, $afterDay)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:getYear"
                 as="xs:string">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:value-of select="substring(dtf:removeTimeZone($dateString), 1, 4)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:getMonth"
                 as="xs:string">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:value-of select="substring(dtf:removeTimeZone($dateString), 6, 2)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:getDay"
                 as="xs:string">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:value-of select="substring(dtf:removeTimeZone($dateString), 9, 2)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:getTimeZone"
                 as="xs:string">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:variable name="dateTimeEndingWithTimezone"
                    as="xs:string"
                    select="concat('^.*(',$timeZoneRegEx,')$')"/>
		    <xsl:choose>
			      <xsl:when test="matches($dateString, $dateTimeEndingWithTimezone)">
				        <xsl:value-of select="replace($dateString, $dateTimeEndingWithTimezone, '$1')"/>
			      </xsl:when>
			      <xsl:otherwise>
				        <xsl:value-of select="$defaultTimeZone"/>
			      </xsl:otherwise>
		    </xsl:choose>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:yearPortionHasFourDigits"
                 as="xs:boolean">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:variable name="dateWithOnlyFourDigitYearAndOptionalTimeZoneRegEx"
                    as="xs:string"
                    select="concat('^\d{4}(',$timeZoneRegEx,')?$')"/>
		    <xsl:variable name="dateStartingWithFourDigitYearRegEx"
                    as="xs:string"
                    select="'^\d{4}-.*$'"/>
		    <xsl:value-of select="matches($dateString, $dateWithOnlyFourDigitYearAndOptionalTimeZoneRegEx) or                            matches($dateString, $dateStartingWithFourDigitYearRegEx)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:removeTimeZone"
                 as="xs:string">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:value-of select="replace($dateString, $timeZoneRegEx, '')"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:padDateTimeWithTemplate"
                 as="xs:dateTime">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:param name="dateTemplateString" as="xs:string"/>
		    <xsl:value-of select="concat($dateString, substring($dateTemplateString, string-length(normalize-space($dateString))+1))"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:isAllowableDateTimeFormat"
                 as="xs:boolean">
		    <xsl:param name="input" as="xs:string"/>
		    <xsl:variable name="trimmedInput" as="xs:string" select="normalize-space($input)"/>

		    
		    <xsl:variable name="YYYY" as="xs:string" select="'^\d{4}(Z|[\+-]\d{2}:\d{2})?$'"/>

		    
		    <xsl:variable name="YYYY-MM"
                    as="xs:string"
                    select="'^\d{4}-\d{2}(Z|[\+-]\d{2}:\d{2})?$'"/>

		    
		    <xsl:variable name="YYYY-MM-DD"
                    as="xs:string"
                    select="'^\d{4}-\d{2}-\d{2}(Z|[\+-]\d{2}:\d{2})?$'"/>

		    
		    <xsl:variable name="YYYY-MM-DDThh-mm"
                    as="xs:string"
                    select="'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}(Z|[\+-]\d{2}:\d{2})?$'"/>

		    
		    <xsl:variable name="YYYY-MM-DDThh-mm-ss"
                    as="xs:string"
                    select="'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(\.\d{1,3})?(Z|[\+-]\d{2}:\d{2})?$'"/>

		    <xsl:value-of select="    matches($trimmedInput, $YYYY) or    matches($trimmedInput, $YYYY-MM) or    matches($trimmedInput, $YYYY-MM-DD) or    matches($trimmedInput, $YYYY-MM-DDThh-mm) or    matches($trimmedInput, $YYYY-MM-DDThh-mm-ss)    "/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:startDate"
                 as="xs:dateTime">
		    <xsl:param name="dateString" as="xs:string"/>
		    <xsl:variable name="timeZonePortion" select="dtf:getTimeZone($dateString)"/>
		    <xsl:variable name="dateTimePortion" select="dtf:removeTimeZone($dateString)"/>
		    <xsl:variable name="outputDate"
                    select="dtf:padDateTimeWithTemplate($dateTimePortion, $startDateTimeTemplate)"/>
		    <xsl:value-of select="concat($outputDate, $timeZonePortion)"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:endDate"
                 as="xs:dateTime">
		    <xsl:param name="input" as="xs:string"/>
		    <xsl:variable name="timeZonePortion" select="dtf:getTimeZone($input)"/>
		    <xsl:variable name="dateTimePortion" select="dtf:removeTimeZone($input)"/>
		    <xsl:variable name="outputDate"
                    select="dtf:padDateTimeWithTemplate($dateTimePortion, $endDateTimeTemplate)"/>
		    <xsl:variable name="outputWithCorrectedDay"
                    select="dtf:replaceDateTimeDay($outputDate, dtf:getMaxDay($outputDate))"/>
		    <xsl:choose>
			      <xsl:when test="dtf:getDay($input)">
				        <xsl:value-of select="concat($outputDate, $timeZonePortion)"/>
			      </xsl:when>
			      <xsl:otherwise>
				        <xsl:value-of select="concat($outputWithCorrectedDay, $timeZonePortion)"/>
			      </xsl:otherwise>
		    </xsl:choose>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:overlaps"
                 as="xs:boolean">
		    <xsl:param name="primary" as="xs:string"/>
		    <xsl:param name="secondary" as="xs:string"/>
		    <xsl:variable name="primaryStart"
                    as="xs:dateTime"
                    select="dtf:startDate($primary)"/>
		    <xsl:variable name="primaryEnd" as="xs:dateTime" select="dtf:endDate($primary)"/>
		    <xsl:variable name="secondaryStart"
                    as="xs:dateTime"
                    select="dtf:startDate($secondary)"/>
		    <xsl:variable name="secondaryEnd"
                    as="xs:dateTime"
                    select="dtf:endDate($secondary)"/>
		    <xsl:value-of select="$primaryStart &lt;= $secondaryEnd and $secondaryStart &lt;= $primaryEnd"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:isBefore"
                 as="xs:boolean">
		    <xsl:param name="primary" as="xs:string"/>
		    <xsl:param name="secondary" as="xs:string"/>
		    <xsl:variable name="primaryEnd" as="xs:dateTime" select="dtf:endDate($primary)"/>
		    <xsl:variable name="secondaryStart"
                    as="xs:dateTime"
                    select="dtf:startDate($secondary)"/>
		    <xsl:value-of select="$primaryEnd &lt; $secondaryStart"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:isAfter"
                 as="xs:boolean">
		    <xsl:param name="primary" as="xs:string"/>
		    <xsl:param name="secondary" as="xs:string"/>
		    <xsl:variable name="primaryStart"
                    as="xs:dateTime"
                    select="dtf:startDate($primary)"/>
		    <xsl:variable name="secondaryEnd"
                    as="xs:dateTime"
                    select="dtf:endDate($secondary)"/>
		    <xsl:value-of select="$secondaryEnd &lt; $primaryStart"/>
	  </xsl:function>
   <xsl:function xmlns:sch="http://purl.oclc.org/dsdl/schematron"
                 name="dtf:compareDateTimeRanges"
                 as="xs:boolean">
		    <xsl:param name="primary" as="xs:string"/>
		    <xsl:param name="operator" as="xs:string"/>
		    <xsl:param name="secondary" as="xs:string"/>
		    <xsl:variable name="primaryAndSecondayYearPortionsHaveFourDigits"
                    as="xs:boolean"
                    select="dtf:yearPortionHasFourDigits($primary) and dtf:yearPortionHasFourDigits($secondary)"/>
		    <xsl:choose>
			      <xsl:when test="$primaryAndSecondayYearPortionsHaveFourDigits">
				        <xsl:variable name="primaryStart"
                          as="xs:dateTime"
                          select="dtf:startDate($primary)"/>
				        <xsl:variable name="primaryEnd" as="xs:dateTime" select="dtf:endDate($primary)"/>
				        <xsl:variable name="secondaryStart"
                          as="xs:dateTime"
                          select="dtf:startDate($secondary)"/>
				        <xsl:variable name="secondaryEnd"
                          as="xs:dateTime"
                          select="dtf:endDate($secondary)"/>
				        <xsl:choose>
					
					
					          <xsl:when test="($operator = 'lt' or $operator = '&lt;') and              (($primaryStart = $primaryEnd and $primaryStart = $secondaryStart) or              ($primaryStart = $primaryEnd and $primaryStart = $secondaryEnd) or              ($secondaryStart = $secondaryEnd and $primaryStart = $secondaryStart))">
						            <xsl:value-of select="false()"/>
					          </xsl:when>

					          
					          
					          <xsl:when test="($operator = 'gt' or $operator = '&gt;') and              (($primaryStart = $primaryEnd and $primaryEnd = $secondaryEnd) or              ($primaryStart = $primaryEnd and $primaryEnd = $secondaryStart) or              ($secondaryStart = $secondaryEnd and $primaryEnd = $secondaryEnd))">
						            <xsl:value-of select="false()"/>
					          </xsl:when>

					          
					          <xsl:when test="$operator = 'lt' or $operator = '&lt;' or $operator = '&lt;='">
						            <xsl:value-of select="dtf:isBefore($primary, $secondary) or dtf:overlaps($primary, $secondary)"/>
					          </xsl:when>

					          
					          <xsl:when test="$operator = 'gt' or $operator = '&gt;' or $operator = '&gt;='">
						            <xsl:value-of select="dtf:isAfter($primary, $secondary) or dtf:overlaps($primary, $secondary)"/>
					          </xsl:when>

					          
					          <xsl:otherwise>
						            <xsl:value-of select="false()"/>
					          </xsl:otherwise>
				        </xsl:choose>
			      </xsl:when>
			      <xsl:otherwise>
				        <xsl:value-of select="false()"/>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:function>

   <!--DEFAULT RULES-->


<!--MODE: SCHEMATRON-SELECT-FULL-PATH-->
<!--This mode can be used to generate an ugly though full XPath for locators-->
<xsl:template match="*" mode="schematron-select-full-path">
      <xsl:apply-templates select="." mode="schematron-get-full-path"/>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-->
<!--This mode can be used to generate an ugly though full XPath for locators-->
<xsl:template match="*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">
            <xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>*:</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>[namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:variable name="preceding"
                    select="count(preceding-sibling::*[local-name()=local-name(current())                                   and namespace-uri() = namespace-uri(current())])"/>
      <xsl:text>[</xsl:text>
      <xsl:value-of select="1+ $preceding"/>
      <xsl:text>]</xsl:text>
   </xsl:template>
   <xsl:template match="@*" mode="schematron-get-full-path">
      <xsl:apply-templates select="parent::*" mode="schematron-get-full-path"/>
      <xsl:text>/</xsl:text>
      <xsl:choose>
         <xsl:when test="namespace-uri()=''">@<xsl:value-of select="name()"/>
         </xsl:when>
         <xsl:otherwise>
            <xsl:text>@*[local-name()='</xsl:text>
            <xsl:value-of select="local-name()"/>
            <xsl:text>' and namespace-uri()='</xsl:text>
            <xsl:value-of select="namespace-uri()"/>
            <xsl:text>']</xsl:text>
         </xsl:otherwise>
      </xsl:choose>
   </xsl:template>

   <!--MODE: SCHEMATRON-FULL-PATH-2-->
<!--This mode can be used to generate prefixed XPath for humans-->
<xsl:template match="node() | @*" mode="schematron-get-full-path-2">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="preceding-sibling::*[name(.)=name(current())]">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="not(self::*)">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
   </xsl:template>
   <!--MODE: SCHEMATRON-FULL-PATH-3-->
<!--This mode can be used to generate prefixed XPath for humans 
	(Top-level element has index)-->
<xsl:template match="node() | @*" mode="schematron-get-full-path-3">
      <xsl:for-each select="ancestor-or-self::*">
         <xsl:text>/</xsl:text>
         <xsl:value-of select="name(.)"/>
         <xsl:if test="parent::*">
            <xsl:text>[</xsl:text>
            <xsl:value-of select="count(preceding-sibling::*[name(.)=name(current())])+1"/>
            <xsl:text>]</xsl:text>
         </xsl:if>
      </xsl:for-each>
      <xsl:if test="not(self::*)">
         <xsl:text/>/@<xsl:value-of select="name(.)"/>
      </xsl:if>
   </xsl:template>

   <!--MODE: GENERATE-ID-FROM-PATH -->
<xsl:template match="/" mode="generate-id-from-path"/>
   <xsl:template match="text()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.text-', 1+count(preceding-sibling::text()), '-')"/>
   </xsl:template>
   <xsl:template match="comment()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.comment-', 1+count(preceding-sibling::comment()), '-')"/>
   </xsl:template>
   <xsl:template match="processing-instruction()" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.processing-instruction-', 1+count(preceding-sibling::processing-instruction()), '-')"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-from-path">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:value-of select="concat('.@', name())"/>
   </xsl:template>
   <xsl:template match="*" mode="generate-id-from-path" priority="-0.5">
      <xsl:apply-templates select="parent::*" mode="generate-id-from-path"/>
      <xsl:text>.</xsl:text>
      <xsl:value-of select="concat('.',name(),'-',1+count(preceding-sibling::*[name()=name(current())]),'-')"/>
   </xsl:template>

   <!--MODE: GENERATE-ID-2 -->
<xsl:template match="/" mode="generate-id-2">U</xsl:template>
   <xsl:template match="*" mode="generate-id-2" priority="2">
      <xsl:text>U</xsl:text>
      <xsl:number level="multiple" count="*"/>
   </xsl:template>
   <xsl:template match="node()" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>n</xsl:text>
      <xsl:number count="node()"/>
   </xsl:template>
   <xsl:template match="@*" mode="generate-id-2">
      <xsl:text>U.</xsl:text>
      <xsl:number level="multiple" count="*"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="string-length(local-name(.))"/>
      <xsl:text>_</xsl:text>
      <xsl:value-of select="translate(name(),':','.')"/>
   </xsl:template>
   <!--Strip characters--><xsl:template match="text()" priority="-1"/>

   <!--SCHEMA SETUP-->
<xsl:template match="/">
      <svrl:schematron-output xmlns:svrl="http://purl.oclc.org/dsdl/svrl" title="" schemaVersion="">
         <xsl:comment>
            <xsl:value-of select="$archiveDirParameter"/>   
		 <xsl:value-of select="$archiveNameParameter"/>  
		 <xsl:value-of select="$fileNameParameter"/>  
		 <xsl:value-of select="$fileDirParameter"/>
         </xsl:comment>
         <svrl:text> This is the root file for the IRM Schematron ruleset. It loads all of the
      required CVEs, declares some variables, and includes all of the Rule .sch files. </svrl:text>
         <svrl:ns-prefix-in-attribute-values uri="http://www.w3.org/2001/XMLSchema" prefix="xsd"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:ism" prefix="ism"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:irm" prefix="irm"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:ntk" prefix="ntk"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:tdf" prefix="tdf"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:id" prefix="icid"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:usagency" prefix="usagency"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:pm" prefix="pm"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:intdis" prefix="intdis"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:mime" prefix="mime"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:icgenc" prefix="genc"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:mil:ces:metadata:ddms:5" prefix="ddms"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:cve" prefix="cve"/>
         <svrl:ns-prefix-in-attribute-values uri="http://www.w3.org/1999/xlink" prefix="xlink"/>
         <svrl:ns-prefix-in-attribute-values uri="http://www.w3.org/1999/XSL/Transform" prefix="xsl"/>
         <svrl:ns-prefix-in-attribute-values uri="date:time:function" prefix="dtf"/>
         <svrl:ns-prefix-in-attribute-values uri="urn:us:gov:ic:irm:xsl:util" prefix="util"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <svrl:text>Abstract rule, which asserts that each date contained within the list
		$dateList has a year value within the range $minYear and $maxYear, inclusive. If any value
		in $dateList is not a valid date format, then we return true because we cannot guarantee the
		value provided is not allowed.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M48"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <svrl:text>Abstract rule, which asserts that the date contained within $dateValue
        has a year value within the range $minYear and $maxYear, inclusive.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M49"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">typeConstraintPatterns</xsl:attribute>
            <xsl:attribute name="name">typeConstraintPatterns</xsl:attribute>
            <svrl:text>Collection of global variables for use in other Schematron rules.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M50"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00002</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00002</xsl:attribute>
            <svrl:text>
      [IRM-ID-00002][Error] For every attribute in the namespace
      [urn:us:gov:ic:irm] or [urn:us:mil:ces:metadata:ddms:5], a non-whitespace
      value must be specified.
      
      Human Readable:
    </svrl:text>
            <svrl:text>
        For each element which specifies an attribute in the IRM or DDMS
        namespace, we make sure that each attribute in the IRM or DDMS
        namespace specifies a non-whitespace value.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M70"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00005</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00005</xsl:attribute>
            <svrl:text>ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639:digraph'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$iso639DigraphList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'   [IRM-ID-00005][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639:digraph] then the value of attribute    @ddms:value must be in CVEnumIRMISO639Digraph.xml and no country code portion may    be specified in the ddms:language element value.      Human Readable: ISO 639 digraph language codes must be in the ISO 639 digraph CVE.    ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M71"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00006</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00006</xsl:attribute>
            <svrl:text>ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-2:trigraph'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$iso639-2TrigraphList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'   [IRM-ID-00006][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639-2:trigraph] then the value of    attribute @ddms:value must be in CVEnumIRMISO639-2Trigraph.xml and no country    code portion may be specified in the ddms:value attribute value.      Human Readable: ISO 639-2 trigraph language codes must be in the ISO 639-2   trigraph CVE.   ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M72"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00007</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00007</xsl:attribute>
            <svrl:text>ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-3:trigraph'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$iso639-3TrigraphList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'   [IRM-ID-00007][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639-3:trigraph] then the value of    attribute @ddms:value must be in CVEnumIRMISO639-3Trigraph.xml and no country code    portion may be specified in the ddms:value attribute value.      Human Readable: ISO 639-3 trigraph language codes must in the the ISO 639-3    trigraph CVE.   ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M73"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00010</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00010</xsl:attribute>
            <svrl:text>
        [IRM-ID-00010][Error] If element ddms:language has the attribute @ddms:qualifier 
        value of [RFC5646] then the language code portion of the @ddms:value attribute 
        value must be in CVEnumIRMISO639Digraph.xml or CVEnumIRMISO639-2Trigraph.xml 
        and the country code portion, if present, must be in CVEnumIRMCoverageISO3166Digraph.xml.
        
        Human Readable: RFC5646 language codes must comply with the RFC by using parts 
        from ISO 639 Digraph or ISO 639-2 Trigraph and ISO 3166 Digraph.
    </svrl:text>
            <svrl:text>
        Finds ddms:language elements and checks its qualifier attribute for a value of
        [RFC5646]. If this value is found it will ensure that the value of the element's ddms:value 
        attribute exists in the CVEnumIRMISO639Digraph.xml or CVEnumIRMISO639-2Trigraph.xml 
        enumeration files represented by the $iso639DigraphList or $iso639-2TrigraphList variables. 
        Country code portions (denoted by '-' separation) must be in the 
        CVEnumIRMCoverageISO3166Digraph.xml enumeration file represented by the 
        $coverageIso3166TrigraphList variable.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M74"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00015</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00015</xsl:attribute>
            <svrl:text>
        [IRM-ID-00015][Error] If element ddms:dates exists without one of the attributes @ddms:created or @ddms:posted
        
        Human Readable: Every ddms:dates element must have at least one of @ddms:created or @ddms:posted.
    </svrl:text>
            <svrl:text>
        This rule checks that for each occurrence of ddms:dates that either @ddms:created or @ddms:posted is specified.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M75"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00016</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00016</xsl:attribute>
            <svrl:text>
        [IRM-ID-00016][Error] The permissible values for the year range are 1901 through the current
        year for attributes @ddms:approvedOn, @ddms:dateProcessed, @ddms:receivedOn, @ddms:infoCutOff, 
        @ddms:posted, and @ddms:created.
        
        Human Readable: Dates must be after 1901 and in the past for @ddms:approvedOn, 
        @ddms:dateProcessed, @ddms:receivedOn, @ddms:infoCutOff, @ddms:posted, and @ddms:created.
    </svrl:text>
            <svrl:text>
        This pattern uses abstract rules to consolidate logic. For attributes, 
        we make sure that each date contained within $dateList has a year value
        within the range $minYear and $maxYear, inclusive.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M76"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00017</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00017</xsl:attribute>
            <svrl:text>
        [IRM-ID-00017][Error] The permissible values for the year range are 
        1901 through 9999 for attribute @ddms:validTil.
        
        Human Readable: @ddms:validTil must be after 1901.
    </svrl:text>
            <svrl:text>
        This pattern uses abstract rules to consolidate logic. For attributes, 
        we make sure that each date contained within $dateList has a year value
        within the range $minYear and $maxYear, inclusive.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M77"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00019</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00019</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn] := an xpath to an element</svrl:text>
            <svrl:text>@ddms:approvedOn := an xpath, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn], to a date to compare
        against all dates in (@ddms:created,                                                 @ddms:posted)</svrl:text>
            <svrl:text>(@ddms:created,                                                 @ddms:posted) := a list of xpaths, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn], each to a
        dates in which to compare against @ddms:approvedOn</svrl:text>
            <svrl:text>'&lt;=' := the equality operator to use for comparing each date in (@ddms:created,                                                 @ddms:posted)
        to @ddms:approvedOn</svrl:text>
            <svrl:text>First, we make sure that the primaryDate is an allowable date format. If
        the primary date is not a valid date format, then we return true because we cannot guarantee
        the value provided is not allowed. Then, for each date in (@ddms:created,                                                 @ddms:posted) we perform the
        same check for a valid date format and compare the secondaryDate to the primaryDate. To
        perform comparisons between dates, we use the comparison operator contained in the param
        '&lt;=' and make sure that all comparisons between primary and secondary dates returns
        true. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M78"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00020</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00020</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:infoCutOff] := an xpath to an element</svrl:text>
            <svrl:text>@ddms:infoCutOff := an xpath, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:infoCutOff], to a date to compare
        against all dates in (@ddms:created,                                                 @ddms:posted)</svrl:text>
            <svrl:text>(@ddms:created,                                                 @ddms:posted) := a list of xpaths, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:infoCutOff], each to a
        dates in which to compare against @ddms:infoCutOff</svrl:text>
            <svrl:text>'&lt;=' := the equality operator to use for comparing each date in (@ddms:created,                                                 @ddms:posted)
        to @ddms:infoCutOff</svrl:text>
            <svrl:text>First, we make sure that the primaryDate is an allowable date format. If
        the primary date is not a valid date format, then we return true because we cannot guarantee
        the value provided is not allowed. Then, for each date in (@ddms:created,                                                 @ddms:posted) we perform the
        same check for a valid date format and compare the secondaryDate to the primaryDate. To
        perform comparisons between dates, we use the comparison operator contained in the param
        '&lt;=' and make sure that all comparisons between primary and secondary dates returns
        true. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M79"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00021</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00021</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil] := an xpath to an element</svrl:text>
            <svrl:text>@ddms:validTil := an xpath, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil], to a date to compare
        against all dates in (@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn)</svrl:text>
            <svrl:text>(@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn) := a list of xpaths, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil], each to a
        dates in which to compare against @ddms:validTil</svrl:text>
            <svrl:text>'&gt;=' := the equality operator to use for comparing each date in (@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn)
        to @ddms:validTil</svrl:text>
            <svrl:text>First, we make sure that the primaryDate is an allowable date format. If
        the primary date is not a valid date format, then we return true because we cannot guarantee
        the value provided is not allowed. Then, for each date in (@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn) we perform the
        same check for a valid date format and compare the secondaryDate to the primaryDate. To
        perform comparisons between dates, we use the comparison operator contained in the param
        '&gt;=' and make sure that all comparisons between primary and secondary dates returns
        true. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M80"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00022</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00022</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage := an xpath to an element</svrl:text>
            <svrl:text>ddms:start := an xpath, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage, to a date to compare
        against all dates in (ddms:end)</svrl:text>
            <svrl:text>(ddms:end) := a list of xpaths, relative to tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage, each to a
        dates in which to compare against ddms:start</svrl:text>
            <svrl:text>'&lt;=' := the equality operator to use for comparing each date in (ddms:end)
        to ddms:start</svrl:text>
            <svrl:text>First, we make sure that the primaryDate is an allowable date format. If
        the primary date is not a valid date format, then we return true because we cannot guarantee
        the value provided is not allowed. Then, for each date in (ddms:end) we perform the
        same check for a valid date format and compare the secondaryDate to the primaryDate. To
        perform comparisons between dates, we use the comparison operator contained in the param
        '&lt;=' and make sure that all comparisons between primary and secondary dates returns
        true. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M81"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00023</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00023</xsl:attribute>
            <svrl:text> [IRM-ID-00023][Error] The permissible values for the year range are 0001
        through 9999 for elements ddms:start and ddms:end. 
        
        Human Readable: ddms:start and ddms:end must be positive integers less than 10,000.</svrl:text>
            <svrl:text> This pattern uses abstract rules to consolidate logic. For attributes, we
        make sure that each date contained within $dateList has a year value within the range
        $minYear and $maxYear, inclusive. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M82"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00024</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00024</xsl:attribute>
            <svrl:text>
        [IRM-ID-00024][Warning] For elements ddms:start and ddms:end and attributes 
        @ddms:approvedOn, @ddms:dateProcessed, @ddms:receivedOn, @ddms:infoCutOff, 
        @ddms:posted, @ddms:validTil, and @ddms:created, if the time designator (T) 
        is specified, it is recommended that time zone be specified.
        
        Human Readable: For elements and attributes of date-time types, if the 
        time designator (T) is specified, it is recommended that time zone be specified.
    </svrl:text>
            <svrl:text>
        The pattern applies to ddms:start and ddms:end elements, as well as any element that 
        contains one or more attributes @ddms:approvedOn, @ddms:infoCutOff, @ddms:posted, and 
        @ddms:created. It joins each of these attribute values, if present, into a larger 
        space-separated string. It then breaks this string into tokens at each space character. 
        If the value of the token contains the time zone designator (T), then it makes sure 
        that the token value matches the regular expression for a timeZone, which is defined 
        in the main schema file (IRM_XML.sch).
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M83"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00025</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00025</xsl:attribute>
            <svrl:text>
        [IRM-ID-00025][Error]  The attribute @ism:excludeFromRollup must not be specified for any element in the
        namespace [urn:us:mil:ces:metadata:ddms:5] in a TDF IRM assertion.
        
        Human readable: DDMS 5.0 elements in IRM TDF assertions are not allowed to be excluded from roll-up 
        because the security markings are now in the TDF assertion statement metadata.
    </svrl:text>
            <svrl:text>
        The attribute @ism:excludeFromRollup must not be specified for any element in the
        namespace [urn:us:mil:ces:metadata:ddms:5] in a TDF IRM assertion.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M84"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00029</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00029</xsl:attribute>
            <svrl:text>
        [IRM-ID-00029][Error]
        If element ddms:geospatialCoverage has attribute @ddms:precedence with a value of [Secondary],
        there must be at least one sibling element ddms:geospatialCoverage for which attribute 
        @ddms:precedence has a value of [Primary].
        
        Human Readable: If a secondary country code is provided, there must also be a primary country code. 
    </svrl:text>
            <svrl:text>
        If there is an element geospatialCoverage with attribute precedence
        specified with a value of [Secondary], then we make sure that there
        is at least one sibling geospatialCoverage element with attribute precedence
        specified with a value of [Primary].
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M85"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00030</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00030</xsl:attribute>
            <svrl:text>
        [IRM-ID-00030][Error]
        If attribute @ddms:order is specified with integer value N, there must exist 
        other @ddms:order attributes with values 1 to N-1 with no duplicates.
        
        Human Readable: The values of attribute @ddms:order must be numbered 
        sequentially with no duplicates, beginning at 1.
    </svrl:text>
            <svrl:text>
        A list, named $orderList, is created containing the value of each order 
        attribute within the document after normalizing to remove extra white-space.
        If the total number of items in $orderList does not equal the number 
        of distinct values in $orderList, then a duplicate exists and we return
        false. Otherwise, we make sure that each number from 1 to N, where N 
        is the number of items in $orderList, is contained within $orderList.
        If each number is contained, then we return true. Otherwise, false.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M86"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00033</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00033</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType := the context in which the searchValue exists</svrl:text>
            <svrl:text>. := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$mimeTypeList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'   [IRM-ID-00033][Error] If element ddms:mimeType is specified, it must have a value from CVEnumMIMEType.xml.   ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M87"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00034</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00034</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:language := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:qualifier := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$compoundLanguageQualifierTypeList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'   [IRM-ID-00034][Error] For element ddms:language, attribute ddms:qualifier must have a    value in CVEnumIRMCompoundLanguageQualifierType.xml.   ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M88"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00036</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00036</xsl:attribute>
            <svrl:text>
		[IRM-ID-00036][Error] For any element, if any attribute is specified with the 
		xlink namespace [http://www.w3.org/1999/xlink], then attributes @xlink:type and/or
		@xlink:href must be specified.
		
		Human Readable: If any XLink attributes are specified for an element, then the type
		and/or URL of the link must also be specified.
	</svrl:text>
            <svrl:text>
		Makes sure that for each element that has any attribute in the xlink namespace has
		either xlink:type or xlink:href specified.
	</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M89"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00040</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00040</xsl:attribute>
            <svrl:text>$qualifier := the qualifier value that requires ism to be
        present</svrl:text>
            <svrl:text>'         [IRM-ID-00040][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:irm:activity], then         attribute ism:classification must also be specified.         ' := the error message text to display when the assertion
        fails</svrl:text>
            <svrl:text>This abstract pattern checks that if a particular qualifier is specified
        on a ddms:type element that ism:classification is also specified.</svrl:text>
            <svrl:text>Example usage: &lt;sch:pattern is-a="DdmsTypeIsmEnforcement"
        id="IRM_ID_00039" xmlns:sch="http://purl.oclc.org/dsdl/schematron"&gt; &lt;sch:param
        name="ruleText" value=""/&gt; &lt;sch:param name="codeDesc" value=""/&gt; &lt;sch:param
        name="context" value="ddms:type[@ddms:qualifier=$qualifier]"/&gt; &lt;sch:param name="errMsg"
        value="' [IRM-ID-00039][Error] If ddms:type is specified with a qualifier of
        urn:us:gov:ic:irm:productline then ism:classification must also be specified. '"/&gt;
        &lt;/sch:pattern&gt; Note: $iso4217TrigraphList is defined in the main document,
        IRM_XML.xml.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M90"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00041</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00041</xsl:attribute>
            <svrl:text>$qualifier := the qualifier value that requires ism to be
        present</svrl:text>
            <svrl:text>'         [IRM-ID-00041][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline], then         attribute ism:classification must also be specified.         ' := the error message text to display when the assertion
        fails</svrl:text>
            <svrl:text>This abstract pattern checks that if a particular qualifier is specified
        on a ddms:type element that ism:classification is also specified.</svrl:text>
            <svrl:text>Example usage: &lt;sch:pattern is-a="DdmsTypeIsmEnforcement"
        id="IRM_ID_00039" xmlns:sch="http://purl.oclc.org/dsdl/schematron"&gt; &lt;sch:param
        name="ruleText" value=""/&gt; &lt;sch:param name="codeDesc" value=""/&gt; &lt;sch:param
        name="context" value="ddms:type[@ddms:qualifier=$qualifier]"/&gt; &lt;sch:param name="errMsg"
        value="' [IRM-ID-00039][Error] If ddms:type is specified with a qualifier of
        urn:us:gov:ic:irm:productline then ism:classification must also be specified. '"/&gt;
        &lt;/sch:pattern&gt; Note: $iso4217TrigraphList is defined in the main document,
        IRM_XML.xml.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M91"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00042</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00042</xsl:attribute>
            <svrl:text>$qualifier := the qualifier value that requires ism to be
        present</svrl:text>
            <svrl:text>'         [IRM-ID-00042][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component], then         attribute ism:classification must also be specified.         ' := the error message text to display when the assertion
        fails</svrl:text>
            <svrl:text>This abstract pattern checks that if a particular qualifier is specified
        on a ddms:type element that ism:classification is also specified.</svrl:text>
            <svrl:text>Example usage: &lt;sch:pattern is-a="DdmsTypeIsmEnforcement"
        id="IRM_ID_00039" xmlns:sch="http://purl.oclc.org/dsdl/schematron"&gt; &lt;sch:param
        name="ruleText" value=""/&gt; &lt;sch:param name="codeDesc" value=""/&gt; &lt;sch:param
        name="context" value="ddms:type[@ddms:qualifier=$qualifier]"/&gt; &lt;sch:param name="errMsg"
        value="' [IRM-ID-00039][Error] If ddms:type is specified with a qualifier of
        urn:us:gov:ic:irm:productline then ism:classification must also be specified. '"/&gt;
        &lt;/sch:pattern&gt; Note: $iso4217TrigraphList is defined in the main document,
        IRM_XML.xml.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M92"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00043</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00043</xsl:attribute>
            <svrl:text>$qualifier := the qualifier value that requires ism to be
        present</svrl:text>
            <svrl:text>'         [IRM-ID-00043][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique], then         attribute ism:classification must also be specified.         ' := the error message text to display when the assertion
        fails</svrl:text>
            <svrl:text>This abstract pattern checks that if a particular qualifier is specified
        on a ddms:type element that ism:classification is also specified.</svrl:text>
            <svrl:text>Example usage: &lt;sch:pattern is-a="DdmsTypeIsmEnforcement"
        id="IRM_ID_00039" xmlns:sch="http://purl.oclc.org/dsdl/schematron"&gt; &lt;sch:param
        name="ruleText" value=""/&gt; &lt;sch:param name="codeDesc" value=""/&gt; &lt;sch:param
        name="context" value="ddms:type[@ddms:qualifier=$qualifier]"/&gt; &lt;sch:param name="errMsg"
        value="' [IRM-ID-00039][Error] If ddms:type is specified with a qualifier of
        urn:us:gov:ic:irm:productline then ism:classification must also be specified. '"/&gt;
        &lt;/sch:pattern&gt; Note: $iso4217TrigraphList is defined in the main document,
        IRM_XML.xml.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M93"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00044</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00044</xsl:attribute>
            <svrl:text>$qualifier := the qualifier value that requires ism to be
        present</svrl:text>
            <svrl:text>'         [IRM-ID-00044][Error]         If element ddms:type is specified with a qualifier of         [urn:us:gov:ic:irm:productline] then attribute ism:classification must         also be specified.         ' := the error message text to display when the assertion
        fails</svrl:text>
            <svrl:text>This abstract pattern checks that if a particular qualifier is specified
        on a ddms:type element that ism:classification is also specified.</svrl:text>
            <svrl:text>Example usage: &lt;sch:pattern is-a="DdmsTypeIsmEnforcement"
        id="IRM_ID_00039" xmlns:sch="http://purl.oclc.org/dsdl/schematron"&gt; &lt;sch:param
        name="ruleText" value=""/&gt; &lt;sch:param name="codeDesc" value=""/&gt; &lt;sch:param
        name="context" value="ddms:type[@ddms:qualifier=$qualifier]"/&gt; &lt;sch:param name="errMsg"
        value="' [IRM-ID-00039][Error] If ddms:type is specified with a qualifier of
        urn:us:gov:ic:irm:productline then ism:classification must also be specified. '"/&gt;
        &lt;/sch:pattern&gt; Note: $iso4217TrigraphList is defined in the main document,
        IRM_XML.xml.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M94"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00045</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00045</xsl:attribute>
            <svrl:text>
        [IRM-ID-00045][Error] Element ddms:geospatialCoverage must have ISM 
        classification markings.
        
        Human Readable: The geospatialCoverage element must have a classification attribute.
    </svrl:text>
            <svrl:text>
        For each ddms:geospatialCoverage element, we make sure that attribute
        ism:classification is specified.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M95"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00046</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00046</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$intelDisciplineComponentTechniqueList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00046][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique]         the attribute @ddms:value must be in CVEnumIntelDisciplineComponentTechnique.xml.                  Human Readable: Intel Discipline Component Techniques must be in the          CVEnumIntelDisciplineComponentTechnique CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M96"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00047</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00047</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$intelDisciplineComponentList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00047][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component]         the attribute @ddms:value must be in CVEnumIntelDisciplineComponent.xml.                  Human Readable: Intel Discipline Components must be in the          CVEnumIntelDisciplineComponent CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M97"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00048</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00048</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$intelDisciplineList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00048][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline]         the attribute @ddms:value must be in CVEnumIntelDiscipline.xml.                  Human Readable: Intel Disciplines must be in the          CVEnumIntelDiscipline CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M98"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00050</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00050</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:subject] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:subject := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$productionMetricsSubjectList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00050][Error]         For element ddms:productionsMetrics, attribute @ddms:subject must          be in CVEnumPMSubject.xml.                  Human Readable: Production Metric Subjects must be in the           CVEnumPMSubject CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M99"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00051</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00051</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:coverage] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:coverage := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$productionMetricsCoverageList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00051][Error]         For element ddms:productionsMetrics, attribute @ddms:coverage must          be in CVEnumPMCoverage.xml.                  Human Readable: Production Metric Coverage values must be in the           CVEnumPMCoverage CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M100"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00052</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00052</xsl:attribute>
            <svrl:text>
        [IRM-ID-00052][Error] If element ddms:organization has attribute
        @ddms:acronym specified and the acronym does not contain a country component, then the 
        value must be defined by the USAgency CES. 
        
        Human Readable:
        Utilized agency acronyms with no country component must be defined in the USAgency CES.
    </svrl:text>
            <svrl:text>
        For ddms:organization with @ddms:acronym specified, if @ddms:acronym has no country component,
        then the country value must be defined by the USAgency CES.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M101"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00053</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00053</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:activity'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$activityList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00053][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:activity]         the attribute @ddms:value must be in CVEnumIRMActivity.xml.                  Human Readable: Activity must be in the          CVEnumIRMActivity CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M102"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00054</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00054</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:precedence] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:precedence := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$coveragePrecedenceList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00054][Error]         If element ddms:geospatialCoverage has attribute @ddms:precedence specified,         then the value must be in CVEnumIRMCoveragePrecedence.xml.                  Human Readable: Geospatial Coverage Precedence must be in the CVEnumIRMCoveragePrecedence CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M103"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00055</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00055</xsl:attribute>
            <svrl:text> [IRM-ID-00055][Error] If ddms:geospatialCoverage/@order is specified then
		there must be one and only one of ddms:geospatialIdentifier/ddms:countryCode or
		ddms:geospatialIdentifier/ddms:subDivisionCode. 
		
		Human Readable: A single order value must be
		applied to one country code or one subdivision code but not to both. </svrl:text>
            <svrl:text> Make sure that there is only one ddms:countryCode or order ddms:subDivisionCode when
		ddms:geospatialCoverage uses the order attribute.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M104"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00059</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00059</xsl:attribute>
            <svrl:text>
        [IRM-ID-00059][Error] If MIN_DISCOVERABLE_OR_GREATER, element
        ddms:resource must specify at least one element ddms:subjectCoverage.
        
        Human Readable: If this IRM instance complies with the minimum
        discoverability rules, then the DDMS card must specify a subjectCoverage
        of the referenced resource.
    </svrl:text>
            <svrl:text>
        If MIN_DISCOVERABLE_OR_GREATER, we make sure that element
        ddms:resource specifies at least one element ddms:subjectCoverage.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M105"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00062</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00062</xsl:attribute>
            <svrl:text>
    A valid IC-Identifier must meet the following criteria:
    (1) The id must begin with 'guide://'
    (2) The prefix for the id is an integer up to 16 digits with no leading zeros allowed
    (3) The suffix is an alphanumeric string limited to 36 characters of the set that includes A-Z, a-z, 0-9, underscore, hyphen, and period
    (4) There are no additional characters proceeding the ID.
    In order to determine the provided IC-Identifier meets these criteria, the value parameter is 
    matched against the following regex: ^guide://([1-9][0-9]{0,15}|0)/[A-Za-z0-9_\-\.]{1,36}$.  
  </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M106"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00063</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00063</xsl:attribute>
            <svrl:text>
        [IRM-ID-00063][Error] Element
        ddms:resource/ddms:creator/ddms:organization must specify attribute
        @ddms:acronym.
        
        Human Readable: The DDMS card must specify a creator organization with an IC agency acronym for the referenced resource.
    </svrl:text>
            <svrl:text>
        We make sure that element ddms:resource/ddms:creator/ddms:organization
        exists and specifies attribute @ddms:acronym.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M107"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00064</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00064</xsl:attribute>
            <svrl:text>
        [IRM-ID-00064][Error] Element ddms:resource/ddms:dates must specify
        attribute @ddms:created.
        
        Human Readable: The DDMS card must specify the date on which the
        referenced resource was created.
    </svrl:text>
            <svrl:text>
        We make sure that element ddms:resource/ddms:dates exists and specifies
        attribute @ddms:created.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M108"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00065</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00065</xsl:attribute>
            <svrl:text>
        [IRM-ID-00065][Error] Attribute ddms:resource/ddms:dates/@ddms:created
        must be castable as an xs:dateTime type.
        
        Human Readable: The date on which the referenced resource was created
        must be a dateTime type.
    </svrl:text>
            <svrl:text>
        We make sure that attribute dms:resource/ddms:dates/@ddms:created is
        castable as an xs:dateTime type.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M109"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00068</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00068</xsl:attribute>
            <svrl:text>
        [IRM-ID-00068][Error]
        For element ddms:taskID, if attribute xlink:href exists, then the attribute must 
        have a non-null value.
        
        Human Readable:
    </svrl:text>
            <svrl:text>
        The normalize-spaced value of attribute xlink:href is checked to make sure the length of 
        the resulting string is greater than zero, which indicates non-whitespace content.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M110"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00070</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00070</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:executableindicator'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$executableIndicatorList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00070][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:executableindicator]         the attribute @ddms:value must be in CVEnumIRMExecutableIndicator.xml.                  HHuman Readable: Executable Indicator Value must be in the          CVEnumIRMExecutableIndicator CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M111"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00071</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00071</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:maliciouscodeindicator'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$maliciousCodeIndicatorList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00071][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:maliciouscodeindicator]         the attribute @ddms:value must be in CVEnumIRMMaliciousCodeIndicator.xml.                  Human Readable: Malicious Code Indicator values must be in the          CVEnumIRMMaliciousCodeIndicator CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M112"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00072</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00072</xsl:attribute>
            <svrl:text>
        [IRM-ID-00072][Error] 
        For element ddms:searchableDate, ddms:start must be earlier than ddms:end.
        
        Human Readable: Within the searchableDate element, the date within the start element must be earlier than the date within the end element.
    </svrl:text>
            <svrl:text>
        For each element ddms:searchableDate, the child elements ddms:start and 
        ddms:end are cast as xs:dateTime types, then compared to ensure 
        start is less than end.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M113"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00073</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00073</xsl:attribute>
            <svrl:text>tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:positive:intel'] := the context in which the searchValue exists</svrl:text>
            <svrl:text>@ddms:value := the value which you want to verify is in the list</svrl:text>
            <svrl:text>$positiveIntelList := the list in which to search for the searchValue</svrl:text>
            <svrl:text>'         [IRM-ID-00073][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:positive:intel]         the attribute @ddms:value must be in CVEnumIRMPositiveIntel.xml.                  Human Readable: Positive Intel values must be in the          CVEnumIRMPositiveIntel CVE.         ' := the error message text to display when the assertion fails</svrl:text>
            <svrl:text>This abstract pattern checks to see if an attribute of an element exists in a list.</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M114"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00074</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00074</xsl:attribute>
            <svrl:text>
        [IRM-ID-00074][Error] 
        For element ddms:searchableDate, elements ddms:start and ddms:end must
        match the xsd:dateTime format.
        
        Human Readable: Within the searchableDate element, the start and end elements values must conform to the xsd:dateTime format.
    </svrl:text>
            <svrl:text>
        For each element ddms:searchableDate, we make sure that elements 
        ddms:start and ddms:end are each castable as xs:dateTime type.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M115"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00075</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00075</xsl:attribute>
            <svrl:text>
        [IRM-ID-00075][Error] For element ddms:temporalCoverage, at least one of its child 
        elements, other than ddms:name, must be specified.
        
        Human Readable: Within the temporalCoverage element, at least one of its child elements must not be blank with the exception 
        of the name element.
    </svrl:text>
            <svrl:text>
        This rule ensures that for each ddms:temporalCoverage element one or more 
        child elements contains non-whitespace content by making sure the string
        representation of ddms:temporalCoverage, which includes all children except
        ddms:name, contains non-whitespace content.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M116"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00076</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00076</xsl:attribute>
            <svrl:text>
        [IRM-ID-00076][Error]
        If the ddms:acquiredOn element exists, at least one of its child 
        elements ddms:description, ddms:approximableDate, or ddms:searchableDate must be present.
        
        Human Readable: The acquiredOn element must have at least one of the following child elements: description, 
        approximableDate and searchableDate.
    </svrl:text>
            <svrl:text> 
        For element ddms:acquiredOn, we make sure that one or more of the child
        elements ddms:description, ddms:approximableDate,
        ddms:searchableDate/ddms:start, or ddms:searchableDate/ddms:end 
        is specified with non-whitespace content. 
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M117"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00077</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00077</xsl:attribute>
            <svrl:text>
        [IRM-ID-00077][Error]
        For element ddms:person
        at least one of the following child elements must have non-whitespace content: 
        ddms:surname, ddms:userID, ddms:name, ddms:affiliation, ddms:postalAddress, ddms:phone 
        ddms:email.
        
        Human Readable:
    </svrl:text>
            <svrl:text>
        This pattern uses an abstract rule to consolidate logic. It
        normalizes the space of the value of the specified child elements
        and makes sure that the length of the resulting string is 
        greater than zero, which indicates non-whitespace content.
        
        Element ddms:postalAddress cannot contain text content, so we count the 
        number of its child elements that contain non-white space and make sure
        that the count is great than 0.
        
        The abstract rule is extended once for each required element in 
        rule IRM_ID_00077.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M118"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00078</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00078</xsl:attribute>
            <svrl:text>
		[IRM-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with 
		child element name [infoCutoff], the permissible values for the year range are 1901 
		through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.	
		
		Human Readable: If elements acquiredOn and temporalCoverage have a child element infoCutoff, then the approximableDate, 
		start and end elements must have a year value between 1901 and the current year.
	</svrl:text>
            <svrl:text>
		This pattern uses an abstract rule to consolidate logic.
		It makes sure that the date contained within $dateValue 
		has a year value within the range $minYear and $maxYear,
		inclusive. The abstract rule is extended once for each 
		element required in rule IRM-ID-00078.
	</svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M119"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00079</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00079</xsl:attribute>
            <svrl:text>
        [IRM-ID-00079][Error] The @icid:DESVersion is less than the minimum version 
        allowed: 1. 
        
        Human Readable: The IC-ID version imported by IRM must be greater than or equal to 1. 
    </svrl:text>
            <svrl:text>
        For all irm:ICResourceMetadataPackage elements that contain @icid:DESVersion, we 
        verify that the version is greater than or equal to the minimum allowed version: 1.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M120"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00080</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00080</xsl:attribute>
            <svrl:text>
        [IRM-ID-00080][Error] The @usagency:CESVersion is less than the minimum version 
        allowed: 201502. 
        
        Human Readable: The US Agency version imported by IRM must be greater than or equal to 2015-FEB. 
    </svrl:text>
            <svrl:text>
        For all irm:ICResourceMetadataPackage elements that contain @usagency:CESVersion, we 
        verify that the version is greater than or equal to the minimum allowed version: 201502.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M121"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00081</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00081</xsl:attribute>
            <svrl:text>
        [IRM-ID-00081][Error] A ddms:type element with @ddms:qualifer ProductLine or 
        Intel must not contain any text. 
        
        Human Readable: DDMS Types of ProductLine or Intel must not contain any text within
        the element. 
    </svrl:text>
            <svrl:text>
        For all ddms:type elements that contain a @ddms:qualifier of urn:us:gov:ic:irm:productline 
        or that start with urn:us:gov:ic:cvenum:intdis, we verify that the element does 
        not contain any text.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M122"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00082</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00082</xsl:attribute>
            <svrl:text>
        [IRM-ID-00082][Error] Use of ddms:revisionRecall is forbidden in IRM and a RevRecall assertion should
        be used instead.
        
        Human Readable: Use of the revisionRecall element in DDMS is forbidden with the use of IRM and a Revision/Recall
        assertion should be used instead.
    </svrl:text>
            <svrl:text>
        If there is an IRM assertion, and a peer DDMS assertion has a ddms:revisionRecall element then trigger
        the error.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M123"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00083</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00083</xsl:attribute>
            <svrl:text>
        [IRM-ID-00083][Error]
        If an IRM TrustedDataObject structured statement exists, it must have a DDMS assertion. 
        
        Human Readable: If an IRM Trusted Data Object structured statement exists, it must have a DDMS assertion.  
    </svrl:text>
            <svrl:text>
        If a TDO has an IRM structured statement, then verify that it has a DDMS assertion.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M124"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00084</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00084</xsl:attribute>
            <svrl:text>
        [IRM-ID-00084][Error] If @ddms:subject or @ddms:coverage exists, then @pm:CESVersion must exist as well.
    </svrl:text>
            <svrl:text>
        Make sure that the PM CVE version attribute exists if subject and coverage exists.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M125"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00085</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00085</xsl:attribute>
            <svrl:text>
        [IRM-ID-00085][Error] The @pm:CESVersion is less than the minimum version allowed: 201511. 
        
        Human Readable: The PM version imported by IRM must be greater than or equal to 2015-NOV. 
    </svrl:text>
            <svrl:text>For all elements that contain @pm:CESVersion, this rule verifies that the version
        is greater than or equal to the minimum allowed version: 201511. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M126"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00086</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00086</xsl:attribute>
            <svrl:text>
        [IRM-ID-00086][Error] The @intdis:CESVersion is less than the minimum version allowed: 201609. 
        
        Human Readable: The INTDIS version imported by IRM must be greater than or equal to 2016-SEP. 
    </svrl:text>
            <svrl:text>For all elements that contain @intdis:CESVersion, this rule verifies that the version
        is greater than or equal to the minimum allowed version: 201609. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M127"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00087</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00087</xsl:attribute>
            <svrl:text>
        [IRM-ID-00087][Error] If @ddms:qualifier identifies a intelligence discipline URN, 
        then @intdis:CESVersion must exist as well.
    </svrl:text>
            <svrl:text>
        Make sure that the INTDIS CVE version attribute exists if intelligence discipline resources are identified.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M128"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00088</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00088</xsl:attribute>
            <svrl:text>
        [IRM-ID-00088][Error] The @mime:CESVersion is less than the minimum version allowed: 201609. 
        
        Human Readable: The MIME version imported by IRM must be greater than or equal to 2016-SEP. 
    </svrl:text>
            <svrl:text>For all elements that contain @mime:CESVersion, this rule verifies that the version
        is greater than or equal to the minimum allowed version: 201609. </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M129"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00089</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00089</xsl:attribute>
            <svrl:text>
        [IRM-ID-00089][Error] If @ddms:mimeType exists, then @mime:CESVersion must exist as well.
    </svrl:text>
            <svrl:text>
        Make sure that the MIME CVE version attribute exists if the internet media type of the resource is defined.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M130"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00090</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00090</xsl:attribute>
            <svrl:text>
        [IRM-ID-00090][Error] If subDivisionCode codespace is GENC codespace, then value must be in the GENC subDivisionCode cve.
    </svrl:text>
            <svrl:text>
        If subDivisionCode codespace is GENC codespace, then value must be in the GENC subDivisionCode cve.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M131"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00091</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00091</xsl:attribute>
            <svrl:text>
        [IRM-ID-00091][Warning] Deprecated MIME types should not be used. 
    </svrl:text>
            <svrl:text>
        Deprecated MIME types should not be used.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M132"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00092</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00092</xsl:attribute>
            <svrl:text>
        [IRM-ID-00092][Error] ddms:NonStateActor should contain a value from the NonStateActor CVE
    </svrl:text>
            <svrl:text>
        Make sure that the value within NonStateActor is a value from the CVE.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M133"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00093</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00093</xsl:attribute>
            <svrl:text>
        [IRM-ID-00093][Error] If present, at least one ddms:countryCode in a ddms:geographicIdentifier must be a 
        GENC ED3 codespace: ^ge:GENC:3:(ed3|3-[1-9][0-9]*)$ 
    </svrl:text>
            <svrl:text>
        If present, at least one ddms:countryCode in a ddms:geographicIdentifier must be a GENC ED3 codespace: ^ge:GENC:3:(ed3|3-[1-9][0-9]*)$
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M134"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00094</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00094</xsl:attribute>
            <svrl:text>
        [IRM-ID-00094][Error] If countrycode codespace is GENC codespace, then value must be in the GENC countrycode cve.
    </svrl:text>
            <svrl:text>
        If countrycode codespace is GENC codespace, then value must be in the GENC countrycode cve
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M135"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00095</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00095</xsl:attribute>
            <svrl:text>
        [IRM-ID-00095][Error] If present, at least one ddms:subDivisionCode in a 
        ddms:geographicIdentifier must be a GENC ED3 codespace: ^as:GENC:6:(ed3|3-[1-9][0-9]*)$
    </svrl:text>
            <svrl:text>
        If present, at least one ddms:subDivisionCode in a ddms:geographicIdentifier must be a GENC ED3 codespace: ^as:GENC:6:(ed3|3-[1-9][0-9]*)$
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M136"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00096</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00096</xsl:attribute>
            <svrl:text>
        [IRM-ID-00096][Error] The @ism:DESVersion is less than the minimum version 
        allowed: 13. 
        
        Human Readable: The ISM version imported by IRM must be greater than or equal to 13 
    </svrl:text>
            <svrl:text>
        For all elements that contain @ism:DESVersion, this rule verifies that the version
        is greater than or equal to the minimum allowed version: 13.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M137"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00097</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00097</xsl:attribute>
            <svrl:text>
        [IRM-ID-00097][Error] The @tdf:version is less than the minimum version 
        allowed: 3. 
        
        Human Readable: The TDF version imported by IRM must be greater than or equal to 3. 
    </svrl:text>
            <svrl:text>
        For all elements that contain @tdf:version, this rule verifies that the version
        is greater than or equal to the minimum allowed version: 3.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M138"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00098</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00098</xsl:attribute>
            <svrl:text>
        [IRM-ID-00098][Error] Use of a GENC codespace requires the presence of the IC-GENC CESVersion attribute.
    </svrl:text>
            <svrl:text>
        If a codespace attribute is specified that contains a GENC codespace, 
        then ensure that the IC-GENC CESVersion attribute is specified in the IRM assertion
        on the ICResourceMetadataPackage.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M139"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00099</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00099</xsl:attribute>
            <svrl:text>
        [IRM-ID-00099][Error] The @genc:CESVersion is less than the minimum version 
        allowed: 201609. 
        
        Human Readable: The IC-GENC version imported by IRM must be greater than or equal to 2016-SEP. 
    </svrl:text>
            <svrl:text>
        For all irm:ICResourceMetadataPackage elements that contain @genc:CESVersion, we 
        verify that the version is greater than or equal to the minimum allowed version: 201609.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M140"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00100</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00100</xsl:attribute>
            <svrl:text>
        [IRM-ID-00100][Error] If a ddms:resource element is present in a tdf:StructuredStatement within a tdf:Assertion, then the tdf:Assertion must also contain a tdf:StatementMetadata element.
    </svrl:text>
            <svrl:text>
        If a ddms:resource element is present in a tdf:StructuredStatement within a tdf:Assertion, then the tdf:Assertion must also contain a tdf:StatementMetadata element.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M141"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00101</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00101</xsl:attribute>
            <svrl:text>
        [IRM-ID-00101][Error]  If element ddms:organization has attribute
        @ddms:acronym specified and the acronym begins with "USA:", then the 
        organization value must be defined by the USAgency CES. 
        
        Human Readable:
        Utilized agency acronyms beginning with "USA:" must be defined in the USAgency CES.
    </svrl:text>
            <svrl:text>
        For ddms:organization with @ddms:acronym specified, if @ddms:acronym begins with 'USA:',
        then the country value must be defined by the USAgency CES.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M142"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00102</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00102</xsl:attribute>
            <svrl:text>
        [IRM-ID-00102][Error]  If element ddms:organization has attribute
        @ddms:acronym specified and the acronym contains a country component that is 
        not "USA:", then the country value must be defined by the GENC CES.  
        
        Human Readable:
        Utilized agency acronyms with a country component must have the country defined in the GENC CES.
    </svrl:text>
            <svrl:text>
        For ddms:organization with @ddms:acronym specified, if @ddms:acronym contains a country component which is not 'USA',
        then the country value must be defined by the GENC CES.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M143"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00103</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00103</xsl:attribute>
            <svrl:text>
        [IRM-ID-00103][Error] If element ddms:organization has attribute
        @ddms:acronym specified, then attribute @ddms:acronym must be of type NmToken. 
    </svrl:text>
            <svrl:text>
        For ddms:organization that have @ddms:acronym, @ddms:acronym must be of type NmToken.  
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M144"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00104</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00104</xsl:attribute>
            <svrl:text>
        [IRM-ID-00104][Error] If element ddms:organization has attribute
        @ddms:acronym specified and @ddms:acronym has a country prefix, 
        then the agency suffix after the colon delimiter must be of type NmToken. 
    </svrl:text>
            <svrl:text>
        For ddms:organization that have @ddms:acronym with a country prefix, the agency suffix 
        after the colon delimiter must be of type NmToken.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M145"/>
         <svrl:active-pattern>
            <xsl:attribute name="document">
               <xsl:value-of select="document-uri(/)"/>
            </xsl:attribute>
            <xsl:attribute name="id">IRM-ID-00105</xsl:attribute>
            <xsl:attribute name="name">IRM-ID-00105</xsl:attribute>
            <svrl:text>
        [IRM-ID-00105][Error] Use of the @ddms:qualifier on the ddms:nonStateActor element is required. 
    </svrl:text>
            <svrl:text>
        If the ddms:nonStateActor element is specified within a DDMS assertion affected by an IRM assertion, 
        then verify that the @ddms:qualifier attribute is present on the element.
    </svrl:text>
            <xsl:apply-templates/>
         </svrl:active-pattern>
         <xsl:apply-templates select="/" mode="M146"/>
      </svrl:schematron-output>
   </xsl:template>

   <!--SCHEMATRON PATTERNS-->
<xsl:param name="IRM_COMPLIES_WITH" select="ddms:resource/@ddms:compliesWith"/>
   <xsl:param name="MIN_DISCOVERABLE_OR_GREATER"
              select="util:containsAnyOfTheTokens($IRM_COMPLIES_WITH, ('MIN_DISCOVERABLE'))"/>
   <xsl:param name="MIN_ACCESSIBLE_OR_GREATER"
              select="$MIN_DISCOVERABLE_OR_GREATER or util:containsAnyOfTheTokens($IRM_COMPLIES_WITH, ('MIN_ACCESSIBLE'))"/>
   <xsl:param name="coverageIso3166DigraphList"
              select="document('../../CVE/IRM/CVEnumIRMCoverageISO3166Digraph.xml')//cve:CVE/cve:Enumeration/cve:Term/cve:Value"/>
   <xsl:param name="iso639DigraphList"
              select="document('../../CVE/IRM/CVEnumIRMISO639Digraph.xml')//cve:CVE/cve:Enumeration/cve:Term/cve:Value"/>
   <xsl:param name="iso639-2TrigraphList"
              select="document('../../CVE/IRM/CVEnumIRMISO639-2Trigraph.xml')//cve:CVE/cve:Enumeration/cve:Term/cve:Value"/>
   <xsl:param name="iso639-3TrigraphList"
              select="document('../../CVE/IRM/CVEnumIRMISO639-3Trigraph.xml')//cve:CVE/cve:Enumeration/cve:Term/cve:Value"/>
   <xsl:param name="mimeTypeList"
              select="document('../../CVE/MIME/CVEnumMIMEType.xml')//cve:Value"/>
   <xsl:param name="deprecatedMimeTypeList"
              select="document('../../CVE/MIME/CVEnumMIMEType.xml')/cve:CVE/cve:Enumeration/cve:Term[./@deprecated]/cve:Value"/>
   <xsl:param name="compoundLanguageQualifierTypeList"
              select="document('../../CVE/IRM/CVEnumIRMCompoundLanguageQualifierType.xml')//cve:Value"/>
   <xsl:param name="intelDisciplineComponentTechniqueList"
              select="document('../../CVE/INTDIS/CVEnumIntelDisciplineComponentTechnique.xml')//cve:Value"/>
   <xsl:param name="intelDisciplineComponentList"
              select="document('../../CVE/INTDIS/CVEnumIntelDisciplineComponent.xml')//cve:Value"/>
   <xsl:param name="intelDisciplineList"
              select="document('../../CVE/INTDIS/CVEnumIntelDiscipline.xml')//cve:Value"/>
   <xsl:param name="positiveIntelList"
              select="document('../../CVE/IRM/CVEnumIRMPositiveIntel.xml')//cve:Value"/>
   <xsl:param name="productionMetricsSubjectList"
              select="document('../../CVE/PM/CVEnumPMSubject.xml')//cve:Value"/>
   <xsl:param name="productionMetricsCoverageList"
              select="document('../../CVE/PM/CVEnumPMCoverage.xml')//cve:Value"/>
   <xsl:param name="activityList"
              select="document('../../CVE/IRM/CVEnumIRMActivity.xml')//cve:Value"/>
   <xsl:param name="executableIndicatorList"
              select="document('../../CVE/IRM/CVEnumIRMExecutableIndicator.xml')//cve:Value"/>
   <xsl:param name="maliciousCodeIndicatorList"
              select="document('../../CVE/IRM/CVEnumIRMMaliciousCodeIndicator.xml')//cve:Value"/>
   <xsl:param name="coveragePrecedenceList"
              select="document('../../CVE/IRM/CVEnumIRMCoveragePrecedence.xml')//cve:Value"/>
   <xsl:param name="USAgencyAcronymList"
              select="document('../../CVE/USAgency/CVEnumUSAgencyAcronym.xml')//cve:CVE/cve:Enumeration/cve:Term/cve:Value"/>
   <xsl:param name="nonStateActorsList"
              select="document('../../CVE/PM/CVEnumPMNonStateActors.xml')//cve:Value"/>
   <xsl:param name="gencCountryCodeList"
              select="document('../../CVE/IC-GENC/CVEnumGENCCountryCode.xml')//cve:Value"/>
   <xsl:param name="gencSubDivisionList"
              select="document('../../CVE/IC-GENC/CVEnumGENCSubDivisionCode.xml')//cve:Value"/>
   <xsl:param name="currentYear" select="year-from-dateTime(current-dateTime())"/>
   <xsl:param name="timeZoneRegEx" select="'Z|[\+-]\d{2}:\d{2}'"/>
   <xsl:param name="endsWithTimeZoneRegEx" select="concat('^.*',$timeZoneRegEx,'$')"/>
   <xsl:param name="startDateTimeTemplate" select="'0001-01-01T00:00:00.000'"/>
   <xsl:param name="endDateTimeTemplate" select="'9999-12-01T23:59:59.999'"/>
   <xsl:param name="defaultTimeZone" select="'Z'"/>

   <!--PATTERN -->
<xsl:template match="text()" priority="-1" mode="M48"/>
   <xsl:template match="@*|node()" priority="-2" mode="M48">
      <xsl:apply-templates select="*" mode="M48"/>
   </xsl:template>

   <!--PATTERN -->
<xsl:template match="text()" priority="-1" mode="M49"/>
   <xsl:template match="@*|node()" priority="-2" mode="M49">
      <xsl:apply-templates select="*" mode="M49"/>
   </xsl:template>

   <!--PATTERN typeConstraintPatterns-->
<xsl:variable name="NameStartCharPattern" select="':|[A-Z]|_|[a-z]'"/>
   <xsl:variable name="NameCharPattern"
                 select="concat($NameStartCharPattern, '|-|\.|[0-9]')"/>
   <xsl:variable name="NmTokenPattern" select="concat('(', $NameCharPattern, ')+')"/>
   <xsl:variable name="NmTokensPattern"
                 select="concat($NmTokenPattern, '( ', $NmTokenPattern, ')*')"/>
   <xsl:variable name="BooleanPattern" select="'(false|true|0|1)'"/>
   <xsl:variable name="DatePattern"
                 select="'-?([1-9][0-9]{3,}|0[0-9]{3})-(0[1-9]|1[0-2])-(0[1-9]|[12][0-9]|3[01])(Z|(\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00))?'"/>
   <xsl:template match="text()" priority="-1" mode="M50"/>
   <xsl:template match="@*|node()" priority="-2" mode="M50">
      <xsl:apply-templates select="*" mode="M50"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00002-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement[ddms:resource or irm:ICResourceMetadataPackage]//*[@*[namespace-uri()=('urn:us:gov:ic:irm', 'urn:us:mil:ces:metadata:ddms:5')]]"
                 priority="1000"
                 mode="M70">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement[ddms:resource or irm:ICResourceMetadataPackage]//*[@*[namespace-uri()=('urn:us:gov:ic:irm', 'urn:us:mil:ces:metadata:ddms:5')]]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             every $attribute in @*[namespace-uri()=('urn:us:gov:ic:irm', 'urn:us:mil:ces:metadata:ddms:5')] satisfies                 normalize-space(string($attribute))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $attribute in @*[namespace-uri()=('urn:us:gov:ic:irm', 'urn:us:mil:ces:metadata:ddms:5')] satisfies normalize-space(string($attribute))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00002][Error] For every attribute in the namespace
            [urn:us:gov:ic:irm] or [urn:us:mil:ces:metadata:ddms:5], a non-whitespace
            value must be specified.
            
            Human Readable:
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M70"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M70"/>
   <xsl:template match="@*|node()" priority="-2" mode="M70">
      <xsl:apply-templates select="*" mode="M70"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00005-->


	<!--RULE -->
<xsl:template match="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639:digraph']"
                 priority="1000"
                 mode="M71">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639:digraph']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $iso639DigraphList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $iso639DigraphList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'   [IRM-ID-00005][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639:digraph] then the value of attribute    @ddms:value must be in CVEnumIRMISO639Digraph.xml and no country code portion may    be specified in the ddms:language element value.      Human Readable: ISO 639 digraph language codes must be in the ISO 639 digraph CVE.    '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M71"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M71"/>
   <xsl:template match="@*|node()" priority="-2" mode="M71">
      <xsl:apply-templates select="*" mode="M71"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00006-->


	<!--RULE -->
<xsl:template match="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-2:trigraph']"
                 priority="1000"
                 mode="M72">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-2:trigraph']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $iso639-2TrigraphList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $iso639-2TrigraphList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'   [IRM-ID-00006][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639-2:trigraph] then the value of    attribute @ddms:value must be in CVEnumIRMISO639-2Trigraph.xml and no country    code portion may be specified in the ddms:value attribute value.      Human Readable: ISO 639-2 trigraph language codes must be in the ISO 639-2   trigraph CVE.   '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M72"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M72"/>
   <xsl:template match="@*|node()" priority="-2" mode="M72">
      <xsl:apply-templates select="*" mode="M72"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00007-->


	<!--RULE -->
<xsl:template match="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-3:trigraph']"
                 priority="1000"
                 mode="M73">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="ddms:language[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:iso639-3:trigraph']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $iso639-3TrigraphList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $iso639-3TrigraphList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'   [IRM-ID-00007][Error] If element ddms:language has the attribute @ddms:qualifier    value of [urn:us:gov:ic:cvenum:irm:iso639-3:trigraph] then the value of    attribute @ddms:value must be in CVEnumIRMISO639-3Trigraph.xml and no country code    portion may be specified in the ddms:value attribute value.      Human Readable: ISO 639-3 trigraph language codes must in the the ISO 639-3    trigraph CVE.   '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M73"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M73"/>
   <xsl:template match="@*|node()" priority="-2" mode="M73">
      <xsl:apply-templates select="*" mode="M73"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00010-->


	<!--RULE -->
<xsl:template match="ddms:language[@ddms:qualifier='RFC5646']"
                 priority="1000"
                 mode="M74">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="ddms:language[@ddms:qualifier='RFC5646']"/>
      <xsl:variable name="tokens" select="tokenize(@ddms:value,'-')"/>
      <xsl:variable name="primarySubtag" select="$tokens[1]"/>
      <xsl:variable name="secondarySubtag" select="$tokens[2]"/>
      <xsl:variable name="badPrimaryValues"
                    select="             if($primarySubtag                 and (                     index-of($iso639-2TrigraphList,lower-case($primarySubtag))&gt;0 or                    index-of($iso639DigraphList,$primarySubtag)&gt;0))             then null             else $primarySubtag"/>
      <xsl:variable name="badSecondaryValues"
                    select="             if($secondarySubtag and index-of($coverageIso3166DigraphList,$secondarySubtag)&gt;0)             then null             else $secondarySubtag"/>
      <xsl:variable name="badValues"
                    select="string-join(($badPrimaryValues,              $badSecondaryValues), ' ')"/>
      <xsl:variable name="primarySubtagValid"
                    select="             $primarySubtag and count($badPrimaryValues) = 0"/>
      <xsl:variable name="secondarySubtagValid"
                    select="             if(not($secondarySubtag)) then true() else                 count($badSecondaryValues) = 0             "/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$primarySubtagValid and $secondarySubtagValid"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="$primarySubtagValid and $secondarySubtagValid">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00010][Error] If element ddms:language has the attribute @ddms:qualifier 
            value of [RFC5646] then the language code portion of the @ddms:value attribute 
            value must be in CVEnumIRMISO639Digraph.xml or CVEnumIRMISO639-2Trigraph.xml 
            and the country code portion, if present, must be in CVEnumIRMCoverageISO3166Digraph.xml.
            
            Human Readable: RFC5646 language codes must comply with the RFC by using parts 
            from ISO 639 Digraph or ISO 639-2 Trigraph and ISO 3166 Digraph. The following 
            values were found but are not in the CVEs: 
            <xsl:text/>
                  <xsl:value-of select="for $each in tokenize($badValues, ' ') return concat('[',$each,'] ')"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M74"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M74"/>
   <xsl:template match="@*|node()" priority="-2" mode="M74">
      <xsl:apply-templates select="*" mode="M74"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00015-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//ddms:dates"
                 priority="1000"
                 mode="M75">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//ddms:dates"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ddms:created or @ddms:posted"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="@ddms:created or @ddms:posted">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
    	    [IRM-ID-00015][Error] Every ddms:date must have at least one of @ddms:created or @ddms:posted.
    	</svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M75"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M75"/>
   <xsl:template match="@*|node()" priority="-2" mode="M75">
      <xsl:apply-templates select="*" mode="M75"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00016-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn |                        @ddms:dateProcessed |                        @ddms:receivedOn |                        @ddms:posted |                         @ddms:created |                         @ddms:infoCutOff]"
                 priority="1000"
                 mode="M76">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn |                        @ddms:dateProcessed |                        @ddms:receivedOn |                        @ddms:posted |                         @ddms:created |                         @ddms:infoCutOff]"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateList"
                    select="             (string(@ddms:approvedOn),             string(@ddms:dateProcessed),             string(@ddms:receivedOn),             string(@ddms:posted),             string(@ddms:created),              string(@ddms:infoCutOff))"/>
      <xsl:variable name="errMsg"
                    select="'             [IRM-ID-00016][Error] The permissible values for the year range are 1901 through the current             year for attributes @ddms:approvedOn, @ddms:dateProcessed, @ddms:receivedOn, @ddms:infoCutOff,              @ddms:posted, and @ddms:created.                          Human Readable: Dates must be after 1901 and in the past for @ddms:approvedOn,              @ddms:dateProcessed, @ddms:receivedOn, @ddms:infoCutOff, @ddms:posted, and @ddms:created.             '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies     if(not(dtf:isAllowableDateTimeFormat(string($date))))     then true()     else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear     and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies if(not(dtf:isAllowableDateTimeFormat(string($date)))) then true() else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
			               <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
		             </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M76"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M76"/>
   <xsl:template match="@*|node()" priority="-2" mode="M76">
      <xsl:apply-templates select="*" mode="M76"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00017-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil]"
                 priority="1000"
                 mode="M77">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil]"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="9999"/>
      <xsl:variable name="dateList" select="(string(@ddms:validTil))"/>
      <xsl:variable name="errMsg"
                    select="'             [IRM-ID-00017][Error] The permissible values for the year range are              1901 through 9999 for attribute @ddms:validTil.                          Human Readable: @ddms:validTil must be after 1901.             '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies     if(not(dtf:isAllowableDateTimeFormat(string($date))))     then true()     else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear     and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies if(not(dtf:isAllowableDateTimeFormat(string($date)))) then true() else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
			               <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
		             </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M77"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M77"/>
   <xsl:template match="@*|node()" priority="-2" mode="M77">
      <xsl:apply-templates select="*" mode="M77"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00019-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn]"
                 priority="1000"
                 mode="M78">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:approvedOn]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:approvedOn))) then every $secondaryDate in (@ddms:created,                                                 @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:approvedOn), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:approvedOn))) then every $secondaryDate in (@ddms:created, @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:approvedOn), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00019][Warning] @ddms:approvedOn must not be later than @ddms:created and @ddms:posted.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:approvedOn))) then every $secondaryDate in (@ddms:created,                                                 @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:approvedOn), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:approvedOn))) then every $secondaryDate in (@ddms:created, @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:approvedOn), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00019][Warning] @ddms:approvedOn must not be later than @ddms:created and @ddms:posted.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M78"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M78"/>
   <xsl:template match="@*|node()" priority="-2" mode="M78">
      <xsl:apply-templates select="*" mode="M78"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00020-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:infoCutOff]"
                 priority="1000"
                 mode="M79">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:infoCutOff]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:infoCutOff))) then every $secondaryDate in (@ddms:created,                                                 @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:infoCutOff), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:infoCutOff))) then every $secondaryDate in (@ddms:created, @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:infoCutOff), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00020][Error] @ddms:infoCutOff must not be later than @ddms:created, and @ddms:posted.          '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:infoCutOff))) then every $secondaryDate in (@ddms:created,                                                 @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:infoCutOff), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:infoCutOff))) then every $secondaryDate in (@ddms:created, @ddms:posted) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:infoCutOff), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00020][Error] @ddms:infoCutOff must not be later than @ddms:created, and @ddms:posted.          '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M79"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M79"/>
   <xsl:template match="@*|node()" priority="-2" mode="M79">
      <xsl:apply-templates select="*" mode="M79"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00021-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil]"
                 priority="1000"
                 mode="M80">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement/ddms:resource//*[@ddms:validTil]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:validTil))) then every $secondaryDate in (@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:validTil), '&gt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(@ddms:validTil))) then every $secondaryDate in (@ddms:created, @ddms:posted, @ddms:infoCutOff, @ddms:approvedOn) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:validTil), '&gt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00021][Warning] @ddms:validTil must not be earlier than         @ddms:created, @ddms:posted, @ddms:infoCutOff, and @ddms:approvedOn.          '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:validTil))) then every $secondaryDate in (@ddms:created,                                                  @ddms:posted,                                                  @ddms:infoCutOff,                                                 @ddms:approvedOn) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:validTil), '&gt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(@ddms:validTil))) then every $secondaryDate in (@ddms:created, @ddms:posted, @ddms:infoCutOff, @ddms:approvedOn) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(@ddms:validTil), '&gt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00021][Warning] @ddms:validTil must not be earlier than         @ddms:created, @ddms:posted, @ddms:infoCutOff, and @ddms:approvedOn.          '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M80"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M80"/>
   <xsl:template match="@*|node()" priority="-2" mode="M80">
      <xsl:apply-templates select="*" mode="M80"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00022-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage"
                 priority="1000"
                 mode="M81">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(ddms:start))) then every $secondaryDate in (ddms:end) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(ddms:start), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'warning' and dtf:isAllowableDateTimeFormat(string(ddms:start))) then every $secondaryDate in (ddms:end) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(ddms:start), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00022][Error] For any element ddms:temporalCoverage, child element ddms:start must          not be later than child element ddms:end.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(ddms:start))) then every $secondaryDate in (ddms:end) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(ddms:start), '&lt;=', string($secondaryDate)) else true() else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ('error' = 'error' and dtf:isAllowableDateTimeFormat(string(ddms:start))) then every $secondaryDate in (ddms:end) satisfies if(dtf:isAllowableDateTimeFormat(string($secondaryDate))) then dtf:compareDateTimeRanges(string(ddms:start), '&lt;=', string($secondaryDate)) else true() else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00022][Error] For any element ddms:temporalCoverage, child element ddms:start must          not be later than child element ddms:end.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M81"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M81"/>
   <xsl:template match="@*|node()" priority="-2" mode="M81">
      <xsl:apply-templates select="*" mode="M81"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00023-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:start |                         tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:end"
                 priority="1000"
                 mode="M82">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:start |                         tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:end"/>
      <xsl:variable name="minYear" select="0001"/>
      <xsl:variable name="maxYear" select="9999"/>
      <xsl:variable name="dateList" select="string(.)"/>
      <xsl:variable name="errMsg"
                    select="'          [IRM-ID-00023][Error] The permissible values for the year range are 0001 through 9999           for elements ddms:start and ddms:end.          Human Readable: ddms:start and ddms:end must be positive integers less than 10,000.             '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies     if(not(dtf:isAllowableDateTimeFormat(string($date))))     then true()     else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear     and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in tokenize(normalize-space(string-join($dateList, ' ')),' ') satisfies if(not(dtf:isAllowableDateTimeFormat(string($date)))) then true() else xs:integer(substring(string($date), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($date), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
			               <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
		             </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M82"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M82"/>
   <xsl:template match="@*|node()" priority="-2" mode="M82">
      <xsl:apply-templates select="*" mode="M82"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00024-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:start"
                 priority="1002"
                 mode="M83">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:start"/>
      <xsl:variable name="dateList" select="."/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             every $date in $dateList satisfies                 if($date castable as xs:dateTime and contains(string($date),'T'))                 then matches(string($date),$endsWithTimeZoneRegEx)                 else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in $dateList satisfies if($date castable as xs:dateTime and contains(string($date),'T')) then matches(string($date),$endsWithTimeZoneRegEx) else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00024][Warning] For elements and attributes of date-time types, if the 
            time designator (T) is specified, it is recommended that time zone be specified.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M83"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:end"
                 priority="1001"
                 mode="M83">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:end"/>
      <xsl:variable name="dateList" select="."/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             every $date in $dateList satisfies                 if($date castable as xs:dateTime and contains(string($date),'T'))                 then matches(string($date),$endsWithTimeZoneRegEx)                 else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in $dateList satisfies if($date castable as xs:dateTime and contains(string($date),'T')) then matches(string($date),$endsWithTimeZoneRegEx) else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00024][Warning] For elements and attributes of date-time types, if the 
            time designator (T) is specified, it is recommended that time zone be specified.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M83"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//*                         [@ddms:approvedOn |                          @ddms:dateProcessed |                         @ddms:receivedOn |                         @ddms:posted |                          @ddms:created |                          @ddms:infoCutOff |                          @ddms:validTil]"
                 priority="1000"
                 mode="M83">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//*                         [@ddms:approvedOn |                          @ddms:dateProcessed |                         @ddms:receivedOn |                         @ddms:posted |                          @ddms:created |                          @ddms:infoCutOff |                          @ddms:validTil]"/>
      <xsl:variable name="dateList"
                    select="             (@ddms:approvedOn,              @ddms:dateProcessed,             @ddms:receivedOn,             @ddms:posted,             @ddms:created,             @ddms:infoCutOff,             @ddms:validTil)             "/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             every $date in $dateList satisfies                 if($date castable as xs:dateTime and contains(string($date),'T'))                 then matches(string($date),$endsWithTimeZoneRegEx)                 else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="every $date in $dateList satisfies if($date castable as xs:dateTime and contains(string($date),'T')) then matches(string($date),$endsWithTimeZoneRegEx) else true()">
               <xsl:attribute name="flag">warning</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00024][Warning] For elements and attributes of date-time types, if the 
            time designator (T) is specified, it is recommended that time zone be specified.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M83"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M83"/>
   <xsl:template match="@*|node()" priority="-2" mode="M83">
      <xsl:apply-templates select="*" mode="M83"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00025-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement//ddms:*"
                 priority="1000"
                 mode="M84">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement//ddms:*"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not(@ism:excludeFromRollup)"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not(@ism:excludeFromRollup)">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00025][Error]The attribute @ism:excludeFromRollup must not be specified for any element in the
            namespace [urn:us:mil:ces:metadata:ddms:5] in a TDF IRM assertion.
            
            Human readable: DDMS 5.0 elements in IRM TDF assertions are not allowed to be excluded from roll-up 
            because the security markings are now in the TDF assertion statement metadata.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M84"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M84"/>
   <xsl:template match="@*|node()" priority="-2" mode="M84">
      <xsl:apply-templates select="*" mode="M84"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00029-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:precedence='Secondary']"
                 priority="1000"
                 mode="M85">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:precedence='Secondary']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="../ddms:geospatialCoverage[@ddms:precedence='Primary']"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="../ddms:geospatialCoverage[@ddms:precedence='Primary']">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00029][Error]
            If element ddms:geospatialCoverage has attribute @ddms:precedence with a value of [Secondary],
            there must be at least one sibling element ddms:geospatialCoverage for which attribute 
            @ddms:precedence has a value of [Primary].
            
            Human Readable: If a secondary country code is provided, there must also be a primary country code.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M85"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M85"/>
   <xsl:template match="@*|node()" priority="-2" mode="M85">
      <xsl:apply-templates select="*" mode="M85"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00030-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource[//@ddms:order]"
                 priority="1000"
                 mode="M86">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource[//@ddms:order]"/>
      <xsl:variable name="orderList"
                    select="tokenize(string-join(//@ddms:order/normalize-space(), ' '), ' ')"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="(count(distinct-values($orderList)) = count($orderList)                     and (every $index in 1 to count($orderList)                              satisfies index-of($orderList, xs:string($index))))             "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="(count(distinct-values($orderList)) = count($orderList) and (every $index in 1 to count($orderList) satisfies index-of($orderList, xs:string($index))))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00030][Error]
            If attribute @ddms:order is specified with integer value N, there must exist 
            other @ddms:order attributes with values 1 to N-1 with no duplicates.
            
            Human Readable: The values of attribute @ddms:order must be numbered 
            sequentially with no duplicates, beginning at 1.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M86"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M86"/>
   <xsl:template match="@*|node()" priority="-2" mode="M86">
      <xsl:apply-templates select="*" mode="M86"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00033-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"
                 priority="1000"
                 mode="M87">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $mimeTypeList satisfies              $token = . or               matches(., concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $mimeTypeList satisfies $token = . or matches(., concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'   [IRM-ID-00033][Error] If element ddms:mimeType is specified, it must have a value from CVEnumMIMEType.xml.   '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M87"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M87"/>
   <xsl:template match="@*|node()" priority="-2" mode="M87">
      <xsl:apply-templates select="*" mode="M87"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00034-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:language"
                 priority="1000"
                 mode="M88">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:language"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $compoundLanguageQualifierTypeList satisfies              $token = @ddms:qualifier or               matches(@ddms:qualifier, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $compoundLanguageQualifierTypeList satisfies $token = @ddms:qualifier or matches(@ddms:qualifier, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'   [IRM-ID-00034][Error] For element ddms:language, attribute ddms:qualifier must have a    value in CVEnumIRMCompoundLanguageQualifierType.xml.   '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M88"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M88"/>
   <xsl:template match="@*|node()" priority="-2" mode="M88">
      <xsl:apply-templates select="*" mode="M88"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00036-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement[ddms:resource|irm:ICResourceMetadataPackage]//*[@xlink:*]"
                 priority="1000"
                 mode="M89">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]//tdf:StructuredStatement[ddms:resource|irm:ICResourceMetadataPackage]//*[@xlink:*]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="normalize-space(string(@xlink:type))    or normalize-space(string(@xlink:href))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="normalize-space(string(@xlink:type)) or normalize-space(string(@xlink:href))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
			[IRM-ID-00036][Error] For any element, if any attribute is specified with the 
			xlink namespace [http://www.w3.org/1999/xlink], then attributes @xlink:type and/or
			@xlink:href must be specified.
			
			Human Readable: If any XLink attributes are specified for an element, then the type
			and/or URL of the link must also be specified.
		</svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M89"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M89"/>
   <xsl:template match="@*|node()" priority="-2" mode="M89">
      <xsl:apply-templates select="*" mode="M89"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00040-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:activity']"
                 priority="1000"
                 mode="M90">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:activity']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00040][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:irm:activity], then         attribute ism:classification must also be specified.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M90"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M90"/>
   <xsl:template match="@*|node()" priority="-2" mode="M90">
      <xsl:apply-templates select="*" mode="M90"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00041-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline']"
                 priority="1000"
                 mode="M91">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00041][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline], then         attribute ism:classification must also be specified.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M91"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M91"/>
   <xsl:template match="@*|node()" priority="-2" mode="M91">
      <xsl:apply-templates select="*" mode="M91"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00042-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component']"
                 priority="1000"
                 mode="M92">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00042][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component], then         attribute ism:classification must also be specified.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M92"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M92"/>
   <xsl:template match="@*|node()" priority="-2" mode="M92">
      <xsl:apply-templates select="*" mode="M92"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00043-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique']"
                 priority="1000"
                 mode="M93">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00043][Error]         If element ddms:type is specified with a qualifier of          [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique], then         attribute ism:classification must also be specified.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M93"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M93"/>
   <xsl:template match="@*|node()" priority="-2" mode="M93">
      <xsl:apply-templates select="*" mode="M93"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00044-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:irm:productline']"
                 priority="1000"
                 mode="M94">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:irm:productline']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00044][Error]         If element ddms:type is specified with a qualifier of         [urn:us:gov:ic:irm:productline] then attribute ism:classification must         also be specified.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M94"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M94"/>
   <xsl:template match="@*|node()" priority="-2" mode="M94">
      <xsl:apply-templates select="*" mode="M94"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00045-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"
                 priority="1000"
                 mode="M95">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ism:classification"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ism:classification">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00045][Error] Element ddms:geospatialCoverage must have ISM
            classification markings.
            
            Human Readable: The geospatialCoverage element must have a classification attribute.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M95"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M95"/>
   <xsl:template match="@*|node()" priority="-2" mode="M95">
      <xsl:apply-templates select="*" mode="M95"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00046-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique']"
                 priority="1000"
                 mode="M96">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $intelDisciplineComponentTechniqueList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $intelDisciplineComponentTechniqueList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00046][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component:technique]         the attribute @ddms:value must be in CVEnumIntelDisciplineComponentTechnique.xml.                  Human Readable: Intel Discipline Component Techniques must be in the          CVEnumIntelDisciplineComponentTechnique CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M96"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M96"/>
   <xsl:template match="@*|node()" priority="-2" mode="M96">
      <xsl:apply-templates select="*" mode="M96"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00047-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component']"
                 priority="1000"
                 mode="M97">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline:component']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $intelDisciplineComponentList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $intelDisciplineComponentList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00047][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline:component]         the attribute @ddms:value must be in CVEnumIntelDisciplineComponent.xml.                  Human Readable: Intel Discipline Components must be in the          CVEnumIntelDisciplineComponent CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M97"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M97"/>
   <xsl:template match="@*|node()" priority="-2" mode="M97">
      <xsl:apply-templates select="*" mode="M97"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00048-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline']"
                 priority="1000"
                 mode="M98">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:intdis:inteldiscipline']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $intelDisciplineList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $intelDisciplineList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00048][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:intdis:inteldiscipline]         the attribute @ddms:value must be in CVEnumIntelDiscipline.xml.                  Human Readable: Intel Disciplines must be in the          CVEnumIntelDiscipline CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M98"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M98"/>
   <xsl:template match="@*|node()" priority="-2" mode="M98">
      <xsl:apply-templates select="*" mode="M98"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00050-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:subject]"
                 priority="1000"
                 mode="M99">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:subject]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $productionMetricsSubjectList satisfies              $token = @ddms:subject or               matches(@ddms:subject, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $productionMetricsSubjectList satisfies $token = @ddms:subject or matches(@ddms:subject, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00050][Error]         For element ddms:productionsMetrics, attribute @ddms:subject must          be in CVEnumPMSubject.xml.                  Human Readable: Production Metric Subjects must be in the           CVEnumPMSubject CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M99"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M99"/>
   <xsl:template match="@*|node()" priority="-2" mode="M99">
      <xsl:apply-templates select="*" mode="M99"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00051-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:coverage]"
                 priority="1000"
                 mode="M100">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:productionMetric[@ddms:coverage]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $productionMetricsCoverageList satisfies              $token = @ddms:coverage or               matches(@ddms:coverage, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $productionMetricsCoverageList satisfies $token = @ddms:coverage or matches(@ddms:coverage, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00051][Error]         For element ddms:productionsMetrics, attribute @ddms:coverage must          be in CVEnumPMCoverage.xml.                  Human Readable: Production Metric Coverage values must be in the           CVEnumPMCoverage CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M100"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M100"/>
   <xsl:template match="@*|node()" priority="-2" mode="M100">
      <xsl:apply-templates select="*" mode="M100"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00052-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"
                 priority="1000"
                 mode="M101">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if (count(tokenize(normalize-space(@ddms:acronym),':')) = 1) then              (some $token in $USAgencyAcronymList satisfies $token = normalize-space(@ddms:acronym))             else              true() "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if (count(tokenize(normalize-space(@ddms:acronym),':')) = 1) then (some $token in $USAgencyAcronymList satisfies $token = normalize-space(@ddms:acronym)) else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00052][Error] If element ddms:organization has attribute
            @ddms:acronym specified and the acronym does not contain a country component, then the 
            value must be defined by the USAgency CES.  /&gt;
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M101"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M101"/>
   <xsl:template match="@*|node()" priority="-2" mode="M101">
      <xsl:apply-templates select="*" mode="M101"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00053-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:activity']"
                 priority="1000"
                 mode="M102">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:activity']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $activityList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $activityList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00053][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:activity]         the attribute @ddms:value must be in CVEnumIRMActivity.xml.                  Human Readable: Activity must be in the          CVEnumIRMActivity CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M102"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M102"/>
   <xsl:template match="@*|node()" priority="-2" mode="M102">
      <xsl:apply-templates select="*" mode="M102"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00054-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:precedence]"
                 priority="1000"
                 mode="M103">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:precedence]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $coveragePrecedenceList satisfies              $token = @ddms:precedence or               matches(@ddms:precedence, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $coveragePrecedenceList satisfies $token = @ddms:precedence or matches(@ddms:precedence, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00054][Error]         If element ddms:geospatialCoverage has attribute @ddms:precedence specified,         then the value must be in CVEnumIRMCoveragePrecedence.xml.                  Human Readable: Geospatial Coverage Precedence must be in the CVEnumIRMCoveragePrecedence CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M103"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M103"/>
   <xsl:template match="@*|node()" priority="-2" mode="M103">
      <xsl:apply-templates select="*" mode="M103"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00055-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:order]"
                 priority="1000"
                 mode="M104">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage[@ddms:order]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="count(ddms:geographicIdentifier/ddms:countryCode) + count(ddms:geographicIdentifier/ddms:subDivisionCode) = 1"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="count(ddms:geographicIdentifier/ddms:countryCode) + count(ddms:geographicIdentifier/ddms:subDivisionCode) = 1">
               <xsl:attribute name="id">IRM-00055</xsl:attribute>
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
			[IRM-ID-00055][Error] If ddms:geospatialCoverage/@order is specified then
			there must be one and only one of ddms:geospatialIdentifier/ddms:countryCode or
			ddms:geospatialIdentifier/ddms:subDivisionCode. 
			
			Human Readable: A single order value
			must be applied to one country code or one subdivision code 
		</svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M104"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M104"/>
   <xsl:template match="@*|node()" priority="-2" mode="M104">
      <xsl:apply-templates select="*" mode="M104"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00059-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource[util:containsAnyOfTheTokens(@ddms:compliesWith, ('MIN_DISCOVERABLE'))]"
                 priority="1000"
                 mode="M105">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource[util:containsAnyOfTheTokens(@ddms:compliesWith, ('MIN_DISCOVERABLE'))]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ddms:subjectCoverage"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="ddms:subjectCoverage">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00059][Error] If MIN_DISCOVERABLE_OR_GREATER, element
            ddms:resource must specify at least one element ddms:subjectCoverage.
            
            Human Readable: If this IRM instance complies with the minimum
            discoverability rules, then the DDMS card must specify a subjectCoverage
            of the referenced resource.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M105"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M105"/>
   <xsl:template match="@*|node()" priority="-2" mode="M105">
      <xsl:apply-templates select="*" mode="M105"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00062-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:identifier[@ddms:qualifier='IC-ID']"
                 priority="1000"
                 mode="M106">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:identifier[@ddms:qualifier='IC-ID']"/>
      <xsl:variable name="icidRegEx"
                    select="'^guide://([1-9][0-9]{0,15}|0)/[A-Za-z0-9_\-\.]{1,36}$'"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="matches(string(string(@ddms:value)),$icidRegEx)"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="matches(string(string(@ddms:value)),$icidRegEx)">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'[IRM-ID-00062][Error] The value of an IC-ID identifier must follow standardized convention. Human Readable: The IC-ID identifier value has to follow standardized convention.'"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M106"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M106"/>
   <xsl:template match="@*|node()" priority="-2" mode="M106">
      <xsl:apply-templates select="*" mode="M106"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00063-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"
                 priority="1000"
                 mode="M107">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ddms:creator/ddms:organization/@ddms:acronym"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="ddms:creator/ddms:organization/@ddms:acronym">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00063][Error] Element
            ddms:resource/ddms:creator/ddms:organization must specify attribute
            @ddms:acronym.
            
            Human Readable: The DDMS card must specify a creator organization with an IC agency acronym for the referenced resource.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M107"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M107"/>
   <xsl:template match="@*|node()" priority="-2" mode="M107">
      <xsl:apply-templates select="*" mode="M107"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00064-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"
                 priority="1000"
                 mode="M108">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ddms:dates/@ddms:created"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="ddms:dates/@ddms:created">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00064][Error] Element ddms:resource/ddms:dates must specify
            attribute @ddms:created.
            
            Human Readable: The DDMS card must specify the date on which the
            referenced resource was created.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M108"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M108"/>
   <xsl:template match="@*|node()" priority="-2" mode="M108">
      <xsl:apply-templates select="*" mode="M108"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00065-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource/ddms:dates[@ddms:created]"
                 priority="1000"
                 mode="M109">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource/ddms:dates[@ddms:created]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ddms:created castable as xs:dateTime"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="@ddms:created castable as xs:dateTime">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00065][Error] Attribute ddms:resource/ddms:dates/@ddms:created
            must be castable as an xs:dateTime type.
            
            Human Readable: The date on which the referenced resource was created
            must be a dateTime type.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M109"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M109"/>
   <xsl:template match="@*|node()" priority="-2" mode="M109">
      <xsl:apply-templates select="*" mode="M109"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00068-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:taskID[@xlink:href]"
                 priority="1000"
                 mode="M110">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:taskID[@xlink:href]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="normalize-space(string(@xlink:href))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="normalize-space(string(@xlink:href))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00068][Error]
            For element ddms:taskID if attribute xlink:href exists, then the attribute must have 
            a non-null value.
            
            Human Readable:
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M110"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M110"/>
   <xsl:template match="@*|node()" priority="-2" mode="M110">
      <xsl:apply-templates select="*" mode="M110"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00070-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:executableindicator']"
                 priority="1000"
                 mode="M111">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:executableindicator']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $executableIndicatorList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $executableIndicatorList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00070][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:executableindicator]         the attribute @ddms:value must be in CVEnumIRMExecutableIndicator.xml.                  HHuman Readable: Executable Indicator Value must be in the          CVEnumIRMExecutableIndicator CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M111"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M111"/>
   <xsl:template match="@*|node()" priority="-2" mode="M111">
      <xsl:apply-templates select="*" mode="M111"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00071-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:maliciouscodeindicator']"
                 priority="1000"
                 mode="M112">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:maliciouscodeindicator']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $maliciousCodeIndicatorList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $maliciousCodeIndicatorList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00071][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:maliciouscodeindicator]         the attribute @ddms:value must be in CVEnumIRMMaliciousCodeIndicator.xml.                  Human Readable: Malicious Code Indicator values must be in the          CVEnumIRMMaliciousCodeIndicator CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M112"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M112"/>
   <xsl:template match="@*|node()" priority="-2" mode="M112">
      <xsl:apply-templates select="*" mode="M112"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00072-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:searchableDate"
                 priority="1000"
                 mode="M113">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:searchableDate"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if((ddms:start castable as xs:dateTime) and (ddms:end castable as xs:dateTime))             then xs:dateTime(ddms:start) &lt; xs:dateTime(ddms:end)             else false()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if((ddms:start castable as xs:dateTime) and (ddms:end castable as xs:dateTime)) then xs:dateTime(ddms:start) &lt; xs:dateTime(ddms:end) else false()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00072][Error]
            For element ddms:searchableDate, ddms:start must be earlier than ddms:end. 
            
            Human Readable: Within the searchableDate element, the date within the start element must be earlier than the date within the end element.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M113"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M113"/>
   <xsl:template match="@*|node()" priority="-2" mode="M113">
      <xsl:apply-templates select="*" mode="M113"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00073-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:positive:intel']"
                 priority="1000"
                 mode="M114">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[@ddms:qualifier='urn:us:gov:ic:cvenum:irm:positive:intel']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             some $token in $positiveIntelList satisfies              $token = @ddms:value or               matches(@ddms:value, concat('^',$token,'$'))                           "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $positiveIntelList satisfies $token = @ddms:value or matches(@ddms:value, concat('^',$token,'$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="'         [IRM-ID-00073][Error]         If element ddms:type has attribute @ddms:qualifier specified as         [urn:us:gov:ic:cvenum:irm:positive:intel]         the attribute @ddms:value must be in CVEnumIRMPositiveIntel.xml.                  Human Readable: Positive Intel values must be in the          CVEnumIRMPositiveIntel CVE.         '"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M114"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M114"/>
   <xsl:template match="@*|node()" priority="-2" mode="M114">
      <xsl:apply-templates select="*" mode="M114"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00074-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:searchableDate"
                 priority="1000"
                 mode="M115">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:searchableDate"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="(ddms:start castable as xs:dateTime) and (ddms:end castable as xs:dateTime)"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="(ddms:start castable as xs:dateTime) and (ddms:end castable as xs:dateTime)">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00074][Error] 
            For element ddms:searchableDate, elements ddms:start and ddms:end must
            match the xsd:dateTime format. 
            
            Human Readable: Within the searchableDate element,
                            the start and end elements values must conform to the xsd:dateTime format.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M115"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M115"/>
   <xsl:template match="@*|node()" priority="-2" mode="M115">
      <xsl:apply-templates select="*" mode="M115"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00075-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement//ddms:resource//ddms:temporalCoverage"
                 priority="1000"
                 mode="M116">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement//ddms:resource//ddms:temporalCoverage"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="count(descendant::node()[not(self::ddms:name)][normalize-space(string-join(text(),''))]) &gt; 0"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="count(descendant::node()[not(self::ddms:name)][normalize-space(string-join(text(),''))]) &gt; 0">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00075][Error] For element ddms:temporalCoverage, at least one of its child 
            elements, other than ddms:name, must be specified.
            
            Human Readable: Within the temporalCoverage element,
            at least one of it's child elements must not be blank
            with the exception of the name element.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M116"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M116"/>
   <xsl:template match="@*|node()" priority="-2" mode="M116">
      <xsl:apply-templates select="*" mode="M116"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00076-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn"
                 priority="1000"
                 mode="M117">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="             normalize-space(string(ddms:description)) or             normalize-space(string(ddms:approximableDate)) or             normalize-space(string(ddms:searchableDate/ddms:start)) or             normalize-space(string(ddms:searchableDate/ddms:end))              "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="normalize-space(string(ddms:description)) or normalize-space(string(ddms:approximableDate)) or normalize-space(string(ddms:searchableDate/ddms:start)) or normalize-space(string(ddms:searchableDate/ddms:end))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00076][Error]
            If the ddms:acquiredOn element exists, at least one of its child 
            elements ddms:description, ddms:approximableDate, or ddms:searchableDate must be present.
            
            Human Readable: The acquiredOn element must have at least one of the following child elements: description, 
            approximableDate and searchableDate.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M117"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M117"/>
   <xsl:template match="@*|node()" priority="-2" mode="M117">
      <xsl:apply-templates select="*" mode="M117"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00077-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:person"
                 priority="1000"
                 mode="M118">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:person"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ddms:surname[normalize-space(string(text()))]                  or ddms:userID[normalize-space(string(text()))]                  or ddms:name[normalize-space(string(text()))]                  or ddms:affiliation[normalize-space(string(text()))]                  or ddms:phone[normalize-space(string(text()))]                  or ddms:email[normalize-space(string(text()))]                  or (some $token in ancestor::ddms:postalAddress/*/text() satisfies                      normalize-space(string($token)))             "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="ddms:surname[normalize-space(string(text()))] or ddms:userID[normalize-space(string(text()))] or ddms:name[normalize-space(string(text()))] or ddms:affiliation[normalize-space(string(text()))] or ddms:phone[normalize-space(string(text()))] or ddms:email[normalize-space(string(text()))] or (some $token in ancestor::ddms:postalAddress/*/text() satisfies normalize-space(string($token)))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00077][Error]
            For element ddms:person
            at least one of the following child elements must have non-whitespace content: 
            ddms:surname, ddms:userID, ddms:name, ddms:affiliation, ddms:postalAddress, ddms:phone 
            ddms:email.
            
            Human Readable:
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M118"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M118"/>
   <xsl:template match="@*|node()" priority="-2" mode="M118">
      <xsl:apply-templates select="*" mode="M118"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00078-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableStart/ddms:searchableDate/ddms:start"
                 priority="1008"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableStart/ddms:searchableDate/ddms:start"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableStart/ddms:searchableDate/ddms:end"
                 priority="1007"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableStart/ddms:searchableDate/ddms:end"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableEnd/ddms:searchableDate/ddms:start"
                 priority="1006"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableEnd/ddms:searchableDate/ddms:start"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableEnd/ddms:searchableDate/ddms:end"
                 priority="1005"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:approximableEnd/ddms:searchableDate/ddms:end"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:start"
                 priority="1004"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:start"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:end"
                 priority="1003"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:temporalCoverage[ddms:name='infoCutOff']/ddms:end"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:approximableDate"
                 priority="1002"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:approximableDate"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:searchableDate/ddms:start"
                 priority="1001"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:searchableDate/ddms:start"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

	  <!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:searchableDate/ddms:end"
                 priority="1000"
                 mode="M119">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:acquiredOn/ddms:searchableDate/ddms:end"/>
      <xsl:variable name="minYear" select="1901"/>
      <xsl:variable name="maxYear" select="$currentYear"/>
      <xsl:variable name="dateValue" select="."/>
      <xsl:variable name="errMsg"
                    select="'    [PUBS-ID-00078][Error] For elements ddms:acquiredOn and ddms:temporalCoverage with     child element name [infoCutoff], the permissible values for the year range are 1901     through the current year for elements ddms:approximableDate, ddms:start, and ddms:end.      '"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if(not(dtf:yearPortionHasFourDigits(string($dateValue)))) then false() else xs:integer(substring(string($dateValue), 1, 4)) &lt;= $maxYear and xs:integer(substring(string($dateValue), 1, 4)) &gt;= $minYear">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
                  <xsl:text/>
                  <xsl:value-of select="$errMsg"/>
                  <xsl:text/>
               </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M119"/>
   <xsl:template match="@*|node()" priority="-2" mode="M119">
      <xsl:apply-templates select="*" mode="M119"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00079-->


	<!--RULE -->
<xsl:template match="irm:ICResourceMetadataPackage[@icid:DESVersion]"
                 priority="1000"
                 mode="M120">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="irm:ICResourceMetadataPackage[@icid:DESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@icid:DESVersion,'-')) then substring-before(@icid:DESVersion,'-') else @icid:DESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 1"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 1">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00079][Error] The @icid:DESVersion is less than the minimum version 
            allowed: 1. 
            
            Human Readable: The IC-ID version imported by IRM must be greater than or equal to 1.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M120"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M120"/>
   <xsl:template match="@*|node()" priority="-2" mode="M120">
      <xsl:apply-templates select="*" mode="M120"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00080-->


	<!--RULE -->
<xsl:template match="irm:ICResourceMetadataPackage[@usagency:CESVersion]"
                 priority="1000"
                 mode="M121">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="irm:ICResourceMetadataPackage[@usagency:CESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@usagency:CESVersion,'-')) then substring-before(@usagency:CESVersion,'-') else @usagency:CESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 201502"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 201502">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00080][Error] The @usagency:CESVersion is less than the minimum version 
            allowed: 201502. 
            
            Human Readable: The US Agency version imported by IRM must be greater than or equal to 2015-FEB.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M121"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M121"/>
   <xsl:template match="@*|node()" priority="-2" mode="M121">
      <xsl:apply-templates select="*" mode="M121"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00081-->


	<!--RULE -->
<xsl:template match="ddms:type[@ddms:qualifier='urn:us:gov:ic:irm:productline'] | ddms:type[contains(@ddms:qualifier,'urn:us:gov:ic:cvenum:intdis')]"
                 priority="1000"
                 mode="M122">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="ddms:type[@ddms:qualifier='urn:us:gov:ic:irm:productline'] | ddms:type[contains(@ddms:qualifier,'urn:us:gov:ic:cvenum:intdis')]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not(normalize-space(text()))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not(normalize-space(text()))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00081][Error] A ddms:type element with @ddms:qualifer ProductLine or 
            Intel must not contain any text. 
            
            Human Readable: DDMS Types of ProductLine or Intel must not contain any text within
            the element. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M122"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M122"/>
   <xsl:template match="@*|node()" priority="-2" mode="M122">
      <xsl:apply-templates select="*" mode="M122"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00082-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:revisionRecall"
                 priority="1000"
                 mode="M123">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:revisionRecall"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="false()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="false()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00082][Error] Use of ddms:revisionRecall is forbidden in IRM and a RevRecall assertion should
            be used instead.
            
            Human Readable: Use of the revisionRecall element in DDMS is forbidden with the use of IRM and a Revision/Recall
            assertion should be used instead.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M123"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M123"/>
   <xsl:template match="@*|node()" priority="-2" mode="M123">
      <xsl:apply-templates select="*" mode="M123"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00083-->


	<!--RULE -->
<xsl:template match="tdf:TrustedDataObject/tdf:Assertion/tdf:StructuredStatement/*[namespace-uri()='urn:us:gov:ic:irm' and local-name()='ICResourceMetadataPackage']"
                 priority="1000"
                 mode="M124">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:TrustedDataObject/tdf:Assertion/tdf:StructuredStatement/*[namespace-uri()='urn:us:gov:ic:irm' and local-name()='ICResourceMetadataPackage']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ancestor::tdf:TrustedDataObject/tdf:Assertion/tdf:StructuredStatement/*[namespace-uri()='urn:us:mil:ces:metadata:ddms:5' and local-name()='resource']"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="ancestor::tdf:TrustedDataObject/tdf:Assertion/tdf:StructuredStatement/*[namespace-uri()='urn:us:mil:ces:metadata:ddms:5' and local-name()='resource']">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00083][Error]
            If a IRM tdf:TrustedDataObject structured statement exists, then it must have a DDMS assertion. 
            
            Human Readable: If a Trusted Data Object has an IRM structured statement, it must have a DDMS assertion. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M124"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M124"/>
   <xsl:template match="@*|node()" priority="-2" mode="M124">
      <xsl:apply-templates select="*" mode="M124"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00084-->


	<!--RULE -->
<xsl:template match="*[@ddms:subject|@ddms:coverage]" priority="1000" mode="M125">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="*[@ddms:subject|@ddms:coverage]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="//@pm:CESVersion"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="//@pm:CESVersion">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00084][Error] If @ddms:subject or @ddms:coverage exists, then @pm:CESVersion must exist as well.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M125"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M125"/>
   <xsl:template match="@*|node()" priority="-2" mode="M125">
      <xsl:apply-templates select="*" mode="M125"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00085-->


	<!--RULE -->
<xsl:template match="*[@pm:CESVersion]" priority="1000" mode="M126">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl" context="*[@pm:CESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@pm:CESVersion,'-')) then substring-before(@pm:CESVersion,'-') else @pm:CESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 201511"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 201511">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00085][Error] The @pm:CESVersion is less than the minimum version 
            allowed: 201511. 
            
            Human Readable: The PM version imported by IRM must be greater than or equal to 2015-NOV. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M126"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M126"/>
   <xsl:template match="@*|node()" priority="-2" mode="M126">
      <xsl:apply-templates select="*" mode="M126"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00086-->


	<!--RULE -->
<xsl:template match="irm:ICResourceMetadataPackage[@intdis:CESVersion]"
                 priority="1000"
                 mode="M127">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="irm:ICResourceMetadataPackage[@intdis:CESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@intdis:CESVersion,'-')) then substring-before(@intdis:CESVersion,'-') else @intdis:CESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 201609"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 201609">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00086][Error] The @intdis:CESVersion is less than the minimum version 
            allowed: 201609. 
            
            Human Readable: The INTDIS version imported by IRM must be greater than or equal to 2016-SEP. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M127"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M127"/>
   <xsl:template match="@*|node()" priority="-2" mode="M127">
      <xsl:apply-templates select="*" mode="M127"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00087-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[starts-with(@ddms:qualifier,'urn:us:gov:ic:cvenum:intdis:inteldiscipline')]"
                 priority="1000"
                 mode="M128">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:type[starts-with(@ddms:qualifier,'urn:us:gov:ic:cvenum:intdis:inteldiscipline')]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ancestor-or-self::tdf:*//@intdis:CESVersion"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="ancestor-or-self::tdf:*//@intdis:CESVersion">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00087][Error] If @ddms:qualifier identifies a intelligence discipline URN, 
            then @intdis:CESVersion must exist as well.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M128"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M128"/>
   <xsl:template match="@*|node()" priority="-2" mode="M128">
      <xsl:apply-templates select="*" mode="M128"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00088-->


	<!--RULE -->
<xsl:template match="*[@mime:CESVersion]" priority="1000" mode="M129">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl" context="*[@mime:CESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@mime:CESVersion,'-')) then substring-before(@mime:CESVersion,'-') else @mime:CESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 201609"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 201609">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00088][Error] The @mime:CESVersion is less than the minimum version 
            allowed: 201609. 
            
            Human Readable: The MIME version imported by IRM must be greater than or equal to 2016-SEP. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M129"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M129"/>
   <xsl:template match="@*|node()" priority="-2" mode="M129">
      <xsl:apply-templates select="*" mode="M129"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00089-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"
                 priority="1000"
                 mode="M130">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="//@mime:CESVersion"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="//@mime:CESVersion">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00089][Error] If @ddms:mimeType exists, then @mime:CESVersion must exist as well.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M130"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M130"/>
   <xsl:template match="@*|node()" priority="-2" mode="M130">
      <xsl:apply-templates select="*" mode="M130"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00090-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage//ddms:subDivisionCode"
                 priority="1000"
                 mode="M131">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage//ddms:subDivisionCode"/>
      <xsl:variable name="isGENCSubDivision"
                    select="matches(normalize-space(./@ddms:codespace), '^as:GENC:6:(ed3|3-[1-9][0-9]*)$')"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not($isGENCSubDivision) or (some $token in $gencSubDivisionList satisfies $token = normalize-space(./@ddms:code))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not($isGENCSubDivision) or (some $token in $gencSubDivisionList satisfies $token = normalize-space(./@ddms:code))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00090][Error] If subDivisionCode codespace is GENC codespace, then value must be in the GENC subDivisionCode cve.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M131"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M131"/>
   <xsl:template match="@*|node()" priority="-2" mode="M131">
      <xsl:apply-templates select="*" mode="M131"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00091-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"
                 priority="1000"
                 mode="M132">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:mimeType"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not(some $deprecatedMimeTypeValue in $deprecatedMimeTypeList satisfies . = $deprecatedMimeTypeValue)"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not(some $deprecatedMimeTypeValue in $deprecatedMimeTypeList satisfies . = $deprecatedMimeTypeValue)">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00091][Warning] Deprecated MIME types should not be used.  
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M132"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M132"/>
   <xsl:template match="@*|node()" priority="-2" mode="M132">
      <xsl:apply-templates select="*" mode="M132"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00092-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:nonStateActor[@ddms:qualifier='urn:us:gov:ic:cvenum:pm:nonstateactors']"
                 priority="1000"
                 mode="M133">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:nonStateActor[@ddms:qualifier='urn:us:gov:ic:cvenum:pm:nonstateactors']"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="some $token in $nonStateActorsList satisfies $token = normalize-space(./text())"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="some $token in $nonStateActorsList satisfies $token = normalize-space(./text())">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00092][Error] ddms:NonStateActor should contain a value from the NonStateActor CVE
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M133"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M133"/>
   <xsl:template match="@*|node()" priority="-2" mode="M133">
      <xsl:apply-templates select="*" mode="M133"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00093-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"
                 priority="1000"
                 mode="M134">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"/>
      <xsl:variable name="hasCountryCodes"
                    select="count(ddms:geographicIdentifier/ddms:countryCode) &gt; 0"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not($hasCountryCodes) or (some $countryCode in ddms:geographicIdentifier/ddms:countryCode satisfies matches(normalize-space($countryCode/@ddms:codespace), '^ge:GENC:3:(ed3|3-[1-9][0-9]*)$'))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not($hasCountryCodes) or (some $countryCode in ddms:geographicIdentifier/ddms:countryCode satisfies matches(normalize-space($countryCode/@ddms:codespace), '^ge:GENC:3:(ed3|3-[1-9][0-9]*)$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00093][Error] ddms:countryCode must be a GENC ED3 codespace: ^ge:GENC:3:(ed3|3-[1-9][0-9]*)$
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M134"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M134"/>
   <xsl:template match="@*|node()" priority="-2" mode="M134">
      <xsl:apply-templates select="*" mode="M134"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00094-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage//ddms:countryCode"
                 priority="1000"
                 mode="M135">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage//ddms:countryCode"/>
      <xsl:variable name="isGENCCountryCode"
                    select="matches(normalize-space(./@ddms:codespace), '^ge:GENC:3:(ed3|3-[1-9][0-9]*)$')"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not($isGENCCountryCode) or (some $token in $gencCountryCodeList satisfies $token = normalize-space(./@ddms:code))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not($isGENCCountryCode) or (some $token in $gencCountryCodeList satisfies $token = normalize-space(./@ddms:code))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00094][Error] ddms:countryCode must be in GENC countrycode cve.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M135"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M135"/>
   <xsl:template match="@*|node()" priority="-2" mode="M135">
      <xsl:apply-templates select="*" mode="M135"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00095-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"
                 priority="1000"
                 mode="M136">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:geospatialCoverage"/>
      <xsl:variable name="hasSubDivisions"
                    select="count(ddms:geographicIdentifier/ddms:subDivisionCode) &gt; 0"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="not($hasSubDivisions) or (some $subDivisionCode in ddms:geographicIdentifier/ddms:subDivisionCode satisfies matches(normalize-space($subDivisionCode/@ddms:codespace), '^as:GENC:6:(ed3|3-[1-9][0-9]*)$'))"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="not($hasSubDivisions) or (some $subDivisionCode in ddms:geographicIdentifier/ddms:subDivisionCode satisfies matches(normalize-space($subDivisionCode/@ddms:codespace), '^as:GENC:6:(ed3|3-[1-9][0-9]*)$'))">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00095][Error] ddms:subDivisionCode must be a GENC ED3 codespace: ^as:GENC:6:(ed3|3-[1-9][0-9]*)$
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M136"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M136"/>
   <xsl:template match="@*|node()" priority="-2" mode="M136">
      <xsl:apply-templates select="*" mode="M136"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00096-->


	<!--RULE -->
<xsl:template match="*[@ism:DESVersion]" priority="1000" mode="M137">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl" context="*[@ism:DESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@ism:DESVersion,'-')) then substring-before(@ism:DESVersion,'-') else @ism:DESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 13"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 13">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00096][Error] The @ism:DESVersion is less than the minimum version 
            allowed: 13. 
            
            Human Readable: The ISM version imported by IRM must be greater than or equal to 13.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M137"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M137"/>
   <xsl:template match="@*|node()" priority="-2" mode="M137">
      <xsl:apply-templates select="*" mode="M137"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00097-->


	<!--RULE -->
<xsl:template match="*[@tdf:version]" priority="1000" mode="M138">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl" context="*[@tdf:version]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@tdf:version,'-')) then substring-before(@tdf:version,'-') else @tdf:version)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 3"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 3">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00097][Error] The @tdf:version is less than the minimum version 
            allowed: 3. 
            
            Human Readable: The TDF version imported by IRM must be greater than or equal to 3.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M138"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M138"/>
   <xsl:template match="@*|node()" priority="-2" mode="M138">
      <xsl:apply-templates select="*" mode="M138"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00098-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//*[starts-with(@ddms:codespace,'ge:GENC') or starts-with(@ddms:codespace,'as:GENC')]"
                 priority="1000"
                 mode="M139">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//*[starts-with(@ddms:codespace,'ge:GENC') or starts-with(@ddms:codespace,'as:GENC')]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="ancestor-or-self::tdf:*/tdf:Assertion//irm:ICResourceMetadataPackage/@genc:CESVersion"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="ancestor-or-self::tdf:*/tdf:Assertion//irm:ICResourceMetadataPackage/@genc:CESVersion">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00098][Error] Use of a GENC codespace requires the presence of the IC-GENC CESVersion attribute.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M139"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M139"/>
   <xsl:template match="@*|node()" priority="-2" mode="M139">
      <xsl:apply-templates select="*" mode="M139"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00099-->


	<!--RULE -->
<xsl:template match="irm:ICResourceMetadataPackage[@genc:CESVersion]"
                 priority="1000"
                 mode="M140">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="irm:ICResourceMetadataPackage[@genc:CESVersion]"/>
      <xsl:variable name="version"
                    select="number(if (contains(@genc:CESVersion,'-')) then substring-before(@genc:CESVersion,'-') else @genc:CESVersion)"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$version &gt;= 201609"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$version &gt;= 201609">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00099][Error] The @genc:CESVersion is less than the minimum version 
            allowed: 201609. 
            
            Human Readable: The IC-GENC version imported by IRM must be greater than or equal to 2016-SEP. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M140"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M140"/>
   <xsl:template match="@*|node()" priority="-2" mode="M140">
      <xsl:apply-templates select="*" mode="M140"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00100-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"
                 priority="1000"
                 mode="M141">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource"/>
      <xsl:variable name="hasStatementMetadata"
                    select="count(../preceding-sibling::tdf:StatementMetadata) &gt; 0"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="$hasStatementMetadata"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="$hasStatementMetadata">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00100][Error] tdf:Assertion must contain tdf:StatementMetadata when tdf:StructuredStatement contains ddms:resource.
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M141"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M141"/>
   <xsl:template match="@*|node()" priority="-2" mode="M141">
      <xsl:apply-templates select="*" mode="M141"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00101-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"
                 priority="1000"
                 mode="M142">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if (starts-with(normalize-space(@ddms:acronym),'USA:')) then                              (some $token in $USAgencyAcronymList satisfies $token = normalize-space(substring-after( @ddms:acronym, ':')))                           else                                true() "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if (starts-with(normalize-space(@ddms:acronym),'USA:')) then (some $token in $USAgencyAcronymList satisfies $token = normalize-space(substring-after( @ddms:acronym, ':'))) else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00101][Error] If element ddms:organization has attribute
            @ddms:acronym specified and the acronym begins with "USA:", then the 
            organization value must be defined by the USAgency CES. /&gt;
         </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M142"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M142"/>
   <xsl:template match="@*|node()" priority="-2" mode="M142">
      <xsl:apply-templates select="*" mode="M142"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00102-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"
                 priority="1000"
                 mode="M143">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"/>
      <xsl:variable name="CC" select="tokenize(normalize-space(@ddms:acronym),':')[1]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if ((count(tokenize(normalize-space(@ddms:acronym),':')) &gt; 1) and not (matches( $CC, 'USA'))) then                                (some $token in $gencCountryCodeList satisfies $token = normalize-space($CC))                           else                                true() "/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if ((count(tokenize(normalize-space(@ddms:acronym),':')) &gt; 1) and not (matches( $CC, 'USA'))) then (some $token in $gencCountryCodeList satisfies $token = normalize-space($CC)) else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00102][Error]  If element ddms:organization has attribute
            @ddms:acronym specified and the acronym contains a country component that is 
            not "USA:", then the country value must be defined by the GENC CES.  /&gt;
         </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M143"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M143"/>
   <xsl:template match="@*|node()" priority="-2" mode="M143">
      <xsl:apply-templates select="*" mode="M143"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00103-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"
                 priority="1000"
                 mode="M144">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="util:meetsType(@ddms:acronym, $NmTokenPattern)"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="util:meetsType(@ddms:acronym, $NmTokenPattern)">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00103][Error] If element ddms:organization has attribute
            @ddms:acronym specified, then attribute @ddms:acronym must be of type NmToken.  
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M144"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M144"/>
   <xsl:template match="@*|node()" priority="-2" mode="M144">
      <xsl:apply-templates select="*" mode="M144"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00104-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"
                 priority="1000"
                 mode="M145">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:organization[@ddms:acronym]"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="if (count(tokenize(normalize-space(@ddms:acronym),':')) &gt; 1)              then util:meetsType(substring-after(@ddms:acronym,':'), $NmTokenPattern)             else true()"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                                test="if (count(tokenize(normalize-space(@ddms:acronym),':')) &gt; 1) then util:meetsType(substring-after(@ddms:acronym,':'), $NmTokenPattern) else true()">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00104][Error] If element ddms:organization has attribute
            @ddms:acronym specified and @ddms:acronym has a country prefix, 
            then the agency suffix after the colon delimiter must be of type NmToken.  
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M145"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M145"/>
   <xsl:template match="@*|node()" priority="-2" mode="M145">
      <xsl:apply-templates select="*" mode="M145"/>
   </xsl:template>

   <!--PATTERN IRM-ID-00105-->


	<!--RULE -->
<xsl:template match="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:nonStateActor"
                 priority="1000"
                 mode="M146">
      <svrl:fired-rule xmlns:svrl="http://purl.oclc.org/dsdl/svrl"
                       context="tdf:*[tdf:Assertion//irm:ICResourceMetadataPackage]/tdf:Assertion/tdf:StructuredStatement/ddms:resource//ddms:nonStateActor"/>

		    <!--ASSERT -->
<xsl:choose>
         <xsl:when test="@ddms:qualifier"/>
         <xsl:otherwise>
            <svrl:failed-assert xmlns:svrl="http://purl.oclc.org/dsdl/svrl" test="@ddms:qualifier">
               <xsl:attribute name="flag">error</xsl:attribute>
               <xsl:attribute name="location">
                  <xsl:apply-templates select="." mode="schematron-select-full-path"/>
               </xsl:attribute>
               <svrl:text>
            [IRM-ID-00105][Error] Use of the @ddms:qualifier on the ddms:nonStateActor element is required. 
        </svrl:text>
            </svrl:failed-assert>
         </xsl:otherwise>
      </xsl:choose>
      <xsl:apply-templates select="*" mode="M146"/>
   </xsl:template>
   <xsl:template match="text()" priority="-1" mode="M146"/>
   <xsl:template match="@*|node()" priority="-2" mode="M146">
      <xsl:apply-templates select="*" mode="M146"/>
   </xsl:template>
</xsl:stylesheet>
<!--UNCLASSIFIED-->
